#ifndef CAEN_BASE_H
#define CAEN_BASE_H
#include <string>
#include <exception>
#include <bitset>
#include <iostream>
#include <stdint.h>
#include "CAENVMEtypes.h"

namespace caen{
  typedef int32_t comm_handle;
  typedef int32_t dig_handle;
  typedef int32_t vme_handle;

  template <class T>
    T bitgrab(T input, size_t low, size_t high, bool shift=true){
    /// grab the bits from [low:high] and send them to the output
    /// shift by >>low to start bits at the LSB (option turns that off)
    std::bitset<32> mask(0);
    for(size_t i=low; i<=high; i++) mask[i]=1;
    T output=(input& mask.to_ulong());
    if(shift) output=(output>>low);
    return output;
  }

 template <class T>
   T bitpack(T input, size_t low, size_t high, size_t start){
   /// write the bits at [low:high] from the input to output
   /// placing them so [low:high] --> [start:start+N] N=(high-low)
   unsigned int inint=input;
   std::bitset<32> inbits(inint);  
   std::bitset<32> outbits(0);  
   for(size_t i=low; i<=high; i++) {
     size_t j=start+(i-low);
    outbits[j]=inbits[i];
   }
   
   //   std::cout<<outbits.to_string()<<std::endl;

   return outbits.to_ulong();
 }
 
 class caen_exception : public std::exception {
 public:
   caen_exception(const char* huh,int err=0){whatstr=huh; code=err;}
   caen_exception(std::string huh, int err=0){whatstr=huh; code=err;}
   virtual ~caen_exception() throw() {};
   virtual const char* what() const throw(){
     return whatstr.c_str();
   }
   virtual int error_code() const throw(){return code;}
 private:
   int code;
   std::string whatstr;
 };
 
 std::string decode_error(const CVErrorCodes& err);
 
}

#endif
