//
//  CAENChain.h  (W.Badgett)
//
//  Class to configure and readout an optical chain 
//  of CAEN CAEN wave form digitizers
//

#ifndef _CAENChain_h
#define _CAENChain_h

#include <fstream>
#include "CAENDigitizer.h"
#include "CAENDigitizerType.h"
#include "CAENDriver.h"
#include "V1740Driver.h"
#include "V1751Driver.h"
#include "LariatConfiguration.h"

class CAENChain
{
  public:
    enum
    {
      MAX_BOARDS = 8,
      MIXED_CHAIN = 1,
      V1740_CHAIN = 2,
      V1751_CHAIN = 3,
    };

    CAENChain(LariatConfiguration *config_, 
	      CAENConfiguration   *caen_,
	      int chainType_);
    ~CAENChain();
    void print();
    void configure();
    void configurePhysics();
    void configurePedestal();
    void fireSWTrigger();
    bool waitForTrigger();
    uint32_t  readEvent(std::ostream *dataFile);
    void writePedestals(std::ostream *pedestalFile);

  protected:
    int chainType;
    V1740Driver *v1740[MAX_BOARDS];
    V1751Driver *v1751[MAX_BOARDS];
    LariatConfiguration * config;
    CAENConfiguration   * caen;
};


#endif
