//
//  CAENConfiguration.h   (W.Badgett)
//

#ifndef _CAENConfiguration_h
#define _CAENConfiguration_h

#include <iostream>
#include <ConfigurationBase.h>

class CAENConfiguration: public ConfigurationBase
{
  public:
  enum
  {
    MAX_BOARDS = 8
  };

  // JCF, 5/13/14 -- I declare the destructor virtual in order to
  // ensure polymorphic behavior from derived classes (such as
  // V1740Configuration)

  virtual ~CAENConfiguration() {}
  CAENConfiguration(std::string fileName):
      ConfigurationBase(fileName),
      link(0),
      nBoards(0),
      enableReadout(0),
      enableBoardMask(0),
      boardId{0,0,0,0,0,0,0,0},
      enableBoard{0,0,0,0,0,0,0,0},
      recordLength(0),
      postPercent(0),
      maxEventsPerTransfer(0),
      eventsPerInterrupt(0),
      irqWaitTime(0),
      usePedestals(0),
      dacValue(0),
      ioLevel(0),
      nChannels(0),
      triggerPolarity(0),
      extTrgMode(0),
      swTrgMode(0),
      acqMode(0),
      debugLevel(0),
      runSyncMode(0),
      outputSignalMode(0)
    {
      char board[128];
      link            = config.get<int>("CAEN.link");
      enableReadout   = config.get<int>("CAEN.enableReadout");
      enableBoardMask = config.get<int>("CAEN.enableBoardMask");
      for ( int i=0; i<MAX_BOARDS; i++)
      {
	sprintf(board,"CAEN.boardId%d", i);
	boardId[i] = config.get<int>(board);
	enableBoard[i] = ( enableBoardMask >> i ) & 1;
	if ( enableBoard[i] != 0 ) { nBoards++;}
      }
      recordLength         = config.get<int>("CAEN.recordLength");
      maxEventsPerTransfer = config.get<int>("CAEN.maxEventsPerTransfer");
      runSyncMode          = config.get<int>("CAEN.runSyncMode");
      outputSignalMode     = config.get<int>("CAEN.outputSignalMode");
      usePedestals         = config.get<bool>("CAEN.usePedestals");
      dacValue             = config.get<int>("CAEN.dacValue");
      ioLevel              = config.get<int>("CAEN.ioLevel");
      nChannels            = config.get<int>("CAEN.nChannels");
      extTrgMode           = config.get<int>("CAEN.extTrgMode");
      swTrgMode            = config.get<int>("CAEN.swTrgMode");
      acqMode              = config.get<int>("CAEN.acqMode");
      triggerPolarity      = config.get<int>("CAEN.triggerPolarity");
      debugLevel           = config.get<int>("CAEN.debugLevel");
      postPercent          = config.get<int>("CAEN.postPercent");
      eventsPerInterrupt   = config.get<int>("CAEN.eventsPerInterrupt");
      irqWaitTime          = config.get<int>("CAEN.irqWaitTime");
    }

    int  link;
    int  nBoards;
    int  enableReadout;
    int  enableBoardMask;
    int  boardId[MAX_BOARDS];
    bool enableBoard[MAX_BOARDS];
    int  recordLength;
    int  postPercent;
    int  maxEventsPerTransfer;
    int  eventsPerInterrupt;
    int  irqWaitTime;
    bool usePedestals;
    int  dacValue;
    int  ioLevel;
    int  nChannels;
    int  triggerPolarity; 
    int  extTrgMode;
    int  swTrgMode;
    int  acqMode;
    int  debugLevel;
    int  runSyncMode;
    int  outputSignalMode;

    void print(std::ostream & os = std::cout) 
    {
      os << "CAEN Configuration" << std::endl;
      ConfigurationBase::print(os);
      os << "  Link                  " << link << std::endl;
      os << "  nBoards               " << nBoards << std::endl;
      os << "  EnableReadout         " << enableReadout << std::endl;
      os << "  EnableBoardMask      0x" << std::hex << enableBoardMask 
	 << std::dec << std::endl;;
      os << "  RecordLength          " << recordLength << std::endl;
      os << "  UsePedestals          " << usePedestals << std::endl;
      os << "  DacValue              " << dacValue << std::endl;
      os << "  nChannels             " << nChannels << std::endl;
      os << "  MaxEventsPerTransfer  " << maxEventsPerTransfer << std::endl;
      os << "  PostPercent           " << postPercent << "%" << std::endl;
      os << "  IrqWaitTime           " << irqWaitTime << std::endl;
      os << "  EventsPerInterrupt    " << eventsPerInterrupt << std::endl;
      os << "  IOLevel (NIM or TTL)  " << ioLevel << " " 
		<< CAENDecoder::IOLevel((CAEN_DGTZ_IOLevel_t)ioLevel) << std::endl;
      os << "  TriggerPolarity       " << triggerPolarity << " " 
		<< CAENDecoder::TriggerPolarity((CAEN_DGTZ_TriggerPolarity_t)triggerPolarity) << std::endl;
      os << "  ExtTrgMode            " << extTrgMode << " " 
		<< CAENDecoder::TriggerMode((CAEN_DGTZ_TriggerMode_t)extTrgMode) << std::endl;
      os << "  SWTrgMode             " << swTrgMode << " " 
		<< CAENDecoder::TriggerMode((CAEN_DGTZ_TriggerMode_t)swTrgMode) << std::endl;
      os << "  AcqMode               " << acqMode << " " 
		<< CAENDecoder::AcquisitionMode((CAEN_DGTZ_AcqMode_t)acqMode) << std::endl;
      os << "  DebugLevel            " << debugLevel << std::endl;
      for ( int i=0; i<nBoards; i++)
      {
	os << "  BoardId[" << i << "]            " << boardId[i] << 
	  "  EnableBoard " << enableBoard[i] << std::endl;
      }
    }
};

#endif

