//
//  CAENDriver.h  (W.Badgett)
//
//  Base class to configure and readout a
//  oCAEN CAEN digitizer
//

#ifndef _CAENDriver_h
#define _CAENDriver_h

#include <stdio.h>
#include <fstream>
#include <ostream>
#include <initializer_list>
#include "CAENDecoder.h"
#include "CAENDigitizer.h"
#include "CAENDigitizerType.h"
#include "LariatConfiguration.h"

class CAENDriver
{
  public:

  enum 
  {
    MAX_GROUPS = 8,
    ACQUISITION_CONTROL_ADDR = 0x8100,
    ACQUISITION_STATUS_ADDR  = 0x8104,
    BOARD_READY_S  = 8,
    PLL_STATUS_S   = 7,
    PLL_BYPASS_S   = 6,
    CLOCK_SOURCE_S = 5,
    EVENT_FULL_S   = 4,
    EVENT_READY_S  = 3,
    RUN_ENABLED_S  = 2,
  };
  int handle;
  uint32_t boardId;
  CAEN_DGTZ_BoardInfo_t boardInfo;
  bool ok;
  
  ~CAENDriver();
  CAENDriver(LariatConfiguration *config_, 
	     CAENConfiguration *caen_,
	     uint32_t boardChainNumber_,
	     uint32_t boardId_);
  virtual void print(std::ostream & os = std::cout);
  uint32_t readEvent(std::ostream *dataFile);
  bool waitForTrigger();
  void readAcquisitionStatus();
  virtual void fireSWTrigger();
  virtual void readConfiguration();
  virtual void setRecordFormat();
  virtual void configure();
  virtual void configureCommon();
  virtual void configurePedestal();
  virtual void configurePhysics();

  virtual void readPedestals(){};


  virtual void writePedestals(std::ostream *pedestalFile){ 
    std::cout << "JCF, 6/12/14 -- this is a dummy function implemented so lariat-artdaq build environment doesn't complain about the unused argument" << std::endl;
    std::cout << "pedestalFile location in memory is at " << pedestalFile << std::endl;
  }


    // Trigger and Configuration registers
  CAEN_DGTZ_EnaDis_t state;
  CAEN_DGTZ_IRQMode_t mode;
  CAEN_DGTZ_TriggerPolarity_t triggerPolarity;
  CAEN_DGTZ_TriggerMode_t swTriggerMode,extTriggerMode;
  CAEN_DGTZ_ZS_Mode_t zeroMode;
  CAEN_DGTZ_AnalogMonitorOutputMode_t analogMode;
  CAEN_DGTZ_RunSyncMode_t runSyncMode;
  CAEN_DGTZ_OutputSignalMode_t outputSignalMode;
  CAEN_DGTZ_IOLevel_t ioLevel;
  CAEN_DGTZ_AcqMode_t acqMode;
  uint8_t interruptLevel;
  uint16_t eventNumber;
  uint32_t statusId,numEvents;
  uint32_t  recordLength,postPercent;
  uint32_t enableMask;
  uint32_t nChannels;
  uint32_t acquisitionStatus;
  bool boardReady;
  bool pllStatus;
  bool pllBypass;
  bool clockSource;
  bool eventFull;
  bool eventReady;
  bool runEnabled;

 protected:
  LariatConfiguration * config;
  CAENConfiguration * caen;
  int boardChainNumber;
  char *buffer;
  CAEN_DGTZ_UINT16_EVENT_t *event;
  uint32_t size;
  double sum[MAX_UINT16_CHANNEL_SIZE];
  double sumsq[MAX_UINT16_CHANNEL_SIZE];
  int nSamples[MAX_UINT16_CHANNEL_SIZE];
  double gsum[MAX_GROUPS];
  double gsumsq[MAX_GROUPS];
  int nGSamples[MAX_GROUPS];
};


#endif
