//
//  LARASICConfiguration.h   (W.Badgett)
//

#ifndef _LARASICConfiguration_h
#define _LARASICConfiguration_h

#include <ConfigurationBase.h>

class LARASICControl
{
  public:
    std::string name;
    LARASICControl(){}
    uint8_t gain;
    uint8_t filter;
    uint8_t dcOffset;
    uint8_t enableTestInput;
    int testChannel;
    void print(std::ostream & os = std::cout) 
    {
      os << "  " << name << " Plane" << std::endl;
      os << "    Gain              " << (uint16_t)gain            << std::endl;
      os << "    Filter            " << (uint16_t)filter          << std::endl;
      os << "    DCOffset          " << (uint16_t)dcOffset        << std::endl;
      os << "    EnableTestInput   " << (uint16_t)enableTestInput << std::endl;
      os << "    TestChannel       " << testChannel     << std::endl;
    }
};

class LARASICConfiguration: public ConfigurationBase
{
  public:

   ~LARASICConfiguration() {}
    LARASICConfiguration(std::string fileName = 
			 "/opt/lariat-online/config/LARASIC.config"):
      ConfigurationBase(fileName)
    {
      enableReadout   = config.get<uint32_t>("LARASIC.enableReadout");
      deviceName      = config.get<std::string>("LARASIC.deviceName");
      channelScan     = config.get<uint32_t>("LARASIC.channelScan");

      induction.name            = std::string("induction");
      induction.gain            = config.get<uint8_t>("LARASIC.induction.gain");
      induction.filter          = config.get<uint8_t>("LARASIC.induction.filter");
      induction.dcOffset        = config.get<uint8_t>("LARASIC.induction.dcOffset");
      induction.enableTestInput = config.get<uint8_t>("LARASIC.induction.enableTestInput");
      induction.testChannel     = config.get<int>("LARASIC.induction.testChannel");

      collection.name            = std::string("collection");
      collection.gain            = config.get<uint8_t>("LARASIC.collection.gain");
      collection.filter          = config.get<uint8_t>("LARASIC.collection.filter");
      collection.dcOffset        = config.get<uint8_t>("LARASIC.collection.dcOffset");
      collection.enableTestInput = config.get<uint8_t>("LARASIC.collection.enableTestInput");
      collection.testChannel     = config.get<int>("LARASIC.collection.testChannel");
    }

    uint32_t enableReadout;
    std::string deviceName;
    uint32_t channelScan;
    LARASICControl induction;
    LARASICControl collection;

    void print(std::ostream & os = std::cout) 
    {
      os << "LARASIC Configuration" << std::endl;
      ConfigurationBase::print(os);
      os << "  EnableReadout          " << enableReadout    << std::endl;
      os << "  DeviceName             " << deviceName       << std::endl;
      os << "  ChannelScan            " << channelScan      << std::endl;
      induction.print(os);
      collection.print(os);
    }
};

#endif

