//
//  LariatReadout.h   (W.Badgett)
//

#ifndef _LariatReadout_h
#define _LariatReadout_h

#include <atomic>
extern "C"
{
#include <curl/curl.h>
#include "signal.h"
}
#include "V1740Configuration.h"
#include "V1751Configuration.h"
#include "CAENChain.h"
#include "TDCDriver.h"
#include "WUTDriver.h"
#include "V1495Driver.h"
#include "V2718Driver.h"
#include "LARASICDriver.h"
#include "LariatFragment.h"

#include <string>

class LariatReadout
{
  public:
    enum
    {
      RUNTYPE_PHYSICS = 1,
      RUNTYPE_PEDESTAL = 2
    };

    LariatReadout(int runType, int events, bool artdaqMode = false);
    ~LariatReadout();
    void readPedestal(std::ostream * output);

    // JCF, 6/13/14

    // "fileToCheck" is only used if the "artdaqMode" boolean in
    // readSpill is set to true; it's the name of the 0- or 1-character ASCII
    // file which, if it contains a 'q' or a 'Q', will cause readSpill to
    // exit

    int  readSpill(std::ostream * output, std::string fileToCheck = "");
    void configurePedestal();
    void configurePhysics();
    void readConfiguration();
    int  fetchRunNumber();
    void printHelp();
    void configure();
    void lariatReset();
    int runNumber() { return(config->runNumber);}
    int spillNumber;
    bool artdaqMode;
    int nV1740s() { return(config->v1740->nBoards);}
    int nV1751s() { return(config->v1751->nBoards);}
    void fetchIFBeam();
    void writeSpillTrailer(std::ostream * output);
    bool PendingCommand();
    bool PendingCommand(bool value);
    void printNusoft();
    void printHTML(std::ostream & os = std::cout);
 
    static void catchSignal(int signalNumber);
    static sig_atomic_t keepRunning;
    static long long deltaDawn(struct timespec before, struct timespec after);

  protected:
    CAENChain     * v1740;
    CAENChain     * v1751;
    TDCDriver     * tdc;
    WUTDriver     * wut;
    V1495Driver   * trigger;
    V2718Driver   * controller;
    LARASICDriver * larasic;

    LariatConfiguration *config;
    bool spillActive;
    bool runActive;
    bool previousSpillActive;
    long long readoutTime, readoutTimeSquares;
    int countV1740, countV1751, spillV1740, spillV1751;
    double spillSnapshot;
    double spillOffset;
    CURL * curl;
    struct LariatFragment::LariatConfig lariatConfig;
    std::atomic_bool pendingCommand;
    bool channelScan;
};

#endif

