//
//  TDCConfiguration.h   (W.Badgett)
//

#ifndef _TDCConfiguration_h
#define _TDCConfiguration_h

#include <ConfigurationBase.h>

class TDCConfiguration: public ConfigurationBase
{
  public:
  enum
  {
    MAX_BOARDS = 16,
    N_CHANNELS = 64,
    N_CHANNEL_MASKS = 4,
    N_THRESHOLD_REGISTERS = 8
  };

   ~TDCConfiguration() {}
    TDCConfiguration(std::string fileName = "/opt/lariat-online/config/TDC.config"):
      ConfigurationBase(fileName),
      spillFlag(false)
    {
      hostName         = config.get<std::string>("TDC.hostName");
      controllerNumber = config.get<uint8_t>("TDC.controllerNumber");
      port             = config.get<uint32_t>("TDC.port");
      nBoards          = config.get<uint32_t>("TDC.nBoards");
      enableMask       = config.get<uint32_t>("TDC.enableMask");
      for ( int i=0; i<MAX_BOARDS; i++)
      {
	enableTDC[i]   = (( enableMask & ( 1 << i ) ) != 0 ); 
      }
      timeOut          = config.get<uint32_t>("TDC.timeOut");
      pulserEnable     = config.get<uint32_t>("TDC.pulserEnable");
      pulserFrequency  = config.get<uint32_t>("TDC.pulserFrequency");
      spillWidth       = config.get<uint32_t>("TDC.spillWidth");
      interSpillWidth  = config.get<uint32_t>("TDC.interSpillWidth");
      debugLevel       = config.get<uint32_t>("TDC.debugLevel");
      spillFlag        = config.get<bool>("TDC.spillFlag");
      enableReadout    = config.get<bool>("TDC.enableReadout");
      std::string startString = config.get<std::string>("TDC.startSpill");
      std::string endString   = config.get<std::string>("TDC.endSpill");

      sscanf(startString.c_str(),"%2X",&startSpill);
      sscanf(endString.c_str(),  "%2X",&endSpill);
      ignoreSpill             = config.get<bool>("TDC.ignoreSpill");
      outputSelect            = config.get<bool>("TDC.outputSelect");
      gateWidth               = config.get<uint16_t>("TDC.gateWidth");
      pipeLineDelay           = config.get<uint16_t>("TDC.pipeLineDelay");
      threshold               = config.get<uint16_t>("TDC.threshold");
    }

    std::string hostName;
    uint8_t  controllerNumber;
    uint32_t port;
    uint32_t nBoards;
    uint32_t enableMask;
    bool     enableTDC[MAX_BOARDS];
    uint32_t timeOut;
    bool     pulserEnable;
    uint32_t pulserFrequency;
    uint32_t spillWidth;
    uint32_t interSpillWidth;
    uint32_t debugLevel;
    bool     spillFlag;
    bool     enableReadout;
    uint32_t startSpill;
    uint32_t endSpill;
    bool     ignoreSpill;
    bool     outputSelect;
    uint16_t gateWidth;
    uint16_t pipeLineDelay;
    uint16_t threshold;

    void print(std::ostream & os = std::cout) 
    {
      os << "TDC Configuration" << std::endl;
      ConfigurationBase::print(os);
      os << "  HostName               " << hostName << std::endl;
      os << "  ControllerNumber       " << (int)controllerNumber << std::endl;
      os << "  Port                   " << port << std::endl;
      os << "  EnableReadout          " << enableReadout << std::endl;
      os << "  nBoards                " << nBoards << std::endl;
      os << "  EnableMask           0x" << std::hex <<  enableMask 
	 << std::dec << std::endl;
      os << "  TimeOut                " << timeOut << std::endl;
      os << "  PulserEnable           " << pulserEnable << std::endl;
      os << "  PulserFrequency        " << pulserFrequency << " "  << 
	( (double)pulserFrequency * 0.0247 )<< std::endl;
      os << "  SpillWidth             " << spillWidth << std::endl;
      os << "  InterSpillWidth        " << interSpillWidth << std::endl;
      os << "  DebugLevel             " << debugLevel << std::endl;
      os << "  SpillFlag              " << spillFlag << std::endl;
      os << "  StartSpill            $" << std::hex << startSpill << std::endl;
      os << "  EndSpill              $" << endSpill << std::dec << std::endl;
      os << "  IgnoreSpill            " << ignoreSpill << std::endl;
      os << "  OutputSelect           " << outputSelect << std::endl;
      os << "  PipeLineDelay        0x" << std::hex <<  pipeLineDelay 
	 << std::dec << " " << pipeLineDelay << std::endl;
      os << "  GateWidth            0x" << std::hex << gateWidth  
	 << std::dec << " " << gateWidth << std::endl;
      os << "  Threshold            0x" << std::hex << threshold 
	 << std::dec << " " << threshold << std::endl;
    }
};

#endif

