//
//  V1495Configuration.h   (M. Kordosky)
//

#ifndef _V1495Configuration_h
#define _V1495Configuration_h

#include <ConfigurationBase.h>
#include <string>
#include <vector>
#include <map>
#include <iostream>
#include <bitset>

class V1495InputChannel{
 public:
 V1495InputChannel():name(""),number(0),reg(0),counts(0){}
  void print(std::ostream& os=std::cout) const {
    os<<"Channel "<<std::dec<<number<<" : "<<name
      <<" at 0x"<<std::hex<<reg<<std::dec<<" with "<<counts<<" counts"<<std::endl;
  }
  std::string name;
  uint32_t number;
  uint32_t reg;
  uint32_t counts;
};

//typedef std::map<std::string,V1495InputChannel> V1495InputChannels;
typedef std::vector<V1495InputChannel> V1495InputChannels;

class V1495TriggerPattern{

 public:
  V1495TriggerPattern(uint32_t inreg, uint32_t maskreg,uint32_t cntreg,
		      const V1495InputChannels& chans,
		      std::string onstring, std::string offstring);

  void configureBitPatterns(const V1495InputChannels& chans,
			    std::string onstring, std::string offstring);
  void print(std::ostream& os=std::cout);

  uint32_t inreg;
  uint32_t maskreg;
  uint32_t cntreg;
  uint32_t counts;
  std::bitset<32> bitpattern;
  std::bitset<32> bitmask;
  std::string onstring;
  std::string offstring;
  
};

typedef std::vector<V1495TriggerPattern> V1495TriggerPatterns;

class V1495Configuration: public ConfigurationBase
{
  public:

   ~V1495Configuration() {}
   V1495Configuration(std::string fileName="/opt/lariat-online/config/V1495.config");

    int bridgeType;
    uint32_t link;
    uint32_t boardNumber;
    uint32_t debugLevel;
    uint32_t baseAddress;
    uint32_t nChan;
    uint32_t nTrigPat;
    uint32_t nVetoPat;
    bool enableReadout;

    V1495InputChannels inputChannels;    
    V1495TriggerPatterns triggerPatterns;
    V1495TriggerPatterns vetoPatterns;

    uint32_t triggerDelay;
    uint32_t triggerDelayReg;
    uint32_t vetoLength;
    uint32_t vetoLengthReg;

    uint32_t controlReg;
    uint32_t counterWidth;
    uint32_t triggerWidth;
    uint32_t M;

    uint32_t tvMask;
    uint32_t tvMaskReg;
    void print(std::ostream& os=std::cout);

};

#endif

