//
//  V1740Driver.h  (W.Badgett)
//
//  Class to configure and readout an optical chain 
//  of CAEN V1740 wave form digitizers
//

#ifndef _V1740Driver_h
#define _V1740Driver_h

#include <stdio.h>
#include <fstream>
#include <ostream>
#include <initializer_list>
#include "CAENDigitizer.h"
#include "CAENDigitizerType.h"
#include "CAENDriver.h"
#include "LariatConfiguration.h"

class V1740Driver: public CAENDriver
{
  public:

  enum
  {
    SAMPLE_REDUCTION_ADDR   = 0x8044,
  };

  // Possible recordLength values:
  //  Code   Blocks   Samples (record length)
  //    0       1       192k
  //    1       2        96k
  //    2       4        48k
  //    3       8        24k      
  //    4      16        12k
  //    5      32         6k
  //    6      64         3k
  //    7     128       1536
  //    8     256        768
  //    9     512        384
  //   10    1024        192
  //

  virtual ~V1740Driver() {}
  V1740Driver(LariatConfiguration *config_, 
	      CAENConfiguration *caen_,
	      uint32_t boardChainNumber_,
	      uint32_t boardId_):
  CAENDriver(config_,caen_,boardChainNumber_,boardId_){}
  void print(std::ostream & os = std::cout);
  virtual void readConfiguration();
  virtual void configureCommon();
  virtual void configurePedestal();
  virtual void configurePhysics();
  virtual void writePedestals(std::ostream *pedestalFile);
  virtual void readPedestals();

  // Trigger and Configuration registers
  CAEN_DGTZ_TriggerMode_t groupSelfTriggerMode[CAENDriver::MAX_GROUPS];
  uint32_t channelMask[CAENDriver::MAX_GROUPS],threshold[CAENDriver::MAX_GROUPS],offset[CAENDriver::MAX_GROUPS];
  uint32_t group;

  uint32_t sampleReduction;
};


#endif
