//
//  V1751Configuration.h   (W.Badgett)
//

#ifndef _V1751Configuration_h
#define _V1751Configuration_h

#include <boost/property_tree/ptree.hpp>
#include <boost/property_tree/xml_parser.hpp>
#include "CAENConfiguration.h"

class V1751Configuration: public CAENConfiguration
{
  public:

    enum
   {
    MAX_CHANNELS = 8
   };
    
   ~V1751Configuration() {}
    V1751Configuration(): CAENConfiguration("/opt/lariat-online/config/V1751.config")
    {
      char chan[128];
      char dcoffset[128];
      char trigthresh[128];
      channelMask=0;
      nWorkingChans=0;
      
      desMode         = config.get<int>("V1751.desMode");
      trigSource      = config.get<int>("V1751.trigSource");
//       enableChannelMask = config.get<int>("CAEN.enableBoardMask");
      for ( int i=0; i<MAX_CHANNELS; i++)
       {
 	sprintf(chan,"V1751.chan%don", i);
 	int chanon = config.get<int>(chan);
	if(chanon)
	  {
	  channelMask|= (1 << i);
 	  nWorkingChans++;
	  }
      //Get DC  offsets per channel
	sprintf(dcoffset,"V1751.chan%dpedestal", i);
 	DCoffset[i] = config.get<int>(dcoffset);
      //Get DC corrections 	
	sprintf(dcoffset,"V1751.chan%dpedcorr", i);
 	DCcorr[i] = config.get<int>(dcoffset);
	
      //Get Trigger thresholds per channel  
	sprintf(trigthresh,"V1751.chan%dthresh", i);
 	TrigThreshold[i] = config.get<int>(trigthresh);
       }
      
    }

   void print(std::ostream & os = std::cout)
   {
      os << "V1751 CAENConfiguration follows" << std::endl;
      CAENConfiguration::print(os);
      os << "V1751 Configuration" << std::endl;
      os << "  DESMode              " << desMode << " " 
		<< CAENDecoder::EnaDisMode((CAEN_DGTZ_EnaDis_t)desMode) 
		<< std::endl;
      os << "  TrigSource           " << trigSource << std::endl;
    }

    int desMode;
    int trigSource;
    int DCoffset[MAX_CHANNELS];
    int DCcorr[MAX_CHANNELS];
    int TrigThreshold[MAX_CHANNELS];
    int channelMask;
    int nWorkingChans;
};

#endif

