//
//  V1751Driver.h  (W.Badgett)
//
//  Class to configure and readout an optical chain 
//  of CAEN V1751 wave form digitizers
//

#ifndef _V1751Driver_h
#define _V1751Driver_h

#include <stdio.h>
#include <fstream>
#include <initializer_list>
#include "CAENDigitizer.h"
#include "CAENDigitizerType.h"
#include "CAENDriver.h"
#include "LariatConfiguration.h"

class V1751Driver: public CAENDriver
{
  public:

  enum 
  {
    CHANNEL_CONFIG_ADDR     = 0x8000,
    BROADCAST_ADDR          = 0x809C,
    FRONT_PANEL_ADDR        = 0x811C,
    TRIGGER_ADDR            = 0X810C,
    FRONT_PANEL_INPUT       = 0xBC,
    MAX_CHANNELS            = 8,
    TRIGGER_SOURCE_MAJORITY = MAX_CHANNELS + 1,
    TRIGGER_SOURCE_EXTERNAL = MAX_CHANNELS + 2
  };

  virtual ~V1751Driver() {}
  V1751Driver(LariatConfiguration *config_,
	      CAENConfiguration *caen_,
	      uint32_t boardChainNumber_,
	      uint32_t boardId_):
  CAENDriver(config_,caen_,boardChainNumber_,boardId_) {}
    //{ boardId_ = 999; boardId_++; } // John F., 5/3/14 --define boardId_, otherwise compiler complains
  void print(std::ostream & os = std::cout);
  virtual void readConfiguration();
  virtual void configureCommon();
  virtual void configurePedestal();
  virtual void configurePhysics();
  virtual void readPedestals();
  virtual void writePedestals(std::ostream * fileName);

 protected:
  CAEN_DGTZ_EnaDis_t desMode;
  int dcOffset[MAX_CHANNELS+1];
  int channelOnMask; 
  
  
};


#endif
