//
//  WUTConfiguration.h   (W.Badgett)
//

#ifndef _WUTConfiguration_h
#define _WUTConfiguration_h

#include <ConfigurationBase.h>

class WUTConfiguration: public ConfigurationBase
{
  public:
  enum
  {
    N_THRESHOLD = 4, 
    CHANNEL_MASK = 0xFFFF,
    EXPECTED_PATTERN_WORD = 0xABCD1234
  };

   ~WUTConfiguration() {}
    WUTConfiguration(std::string fileName = "/opt/lariat-online/config/WUT.config"):
      ConfigurationBase(fileName)
    {
      bridgeType           = config.get<uint32_t>("WUT.bridgeType");
      link                 = config.get<uint32_t>("WUT.link");
      boardNumber          = config.get<uint32_t>("WUT.boardNumber");
      enableReadout        = config.get<uint32_t>("WUT.enableReadout");
      debugLevel           = config.get<uint32_t>("WUT.debugLevel");
      baseAddress          = config.get<uint32_t>("WUT.baseAddress");
      readoutPeriod        = config.get<uint32_t>("WUT.readoutPeriod");
      calibrationDelay     = config.get<uint32_t>("WUT.calibrationDelay");
      externalTrigger      = config.get<uint32_t>("WUT.externalTrigger");
      disableMask          = config.get<uint32_t>("WUT.disableMask");
      useNim1              = config.get<uint32_t>("WUT.useNim1");
      enableBLT            = config.get<uint32_t>("WUT.enableBLT");
      hitSleep             = config.get<uint32_t>("WUT.hitSleep");
      disableSeparator     = config.get<uint32_t>("WUT.disableSeparator");
      char thresholdName [128];
      for ( uint32_t i=0; i<N_THRESHOLD; i++)
      {
	sprintf(thresholdName,"WUT.threshold%d", i);
	threshold[i] = config.get<uint32_t>(thresholdName);
      }
    }

    uint32_t bridgeType;
    uint32_t link;
    uint32_t enableReadout;
    uint32_t boardNumber;
    uint32_t debugLevel;
    uint32_t baseAddress;
    uint32_t readoutPeriod;
    uint32_t calibrationDelay;
    uint32_t calibrationAutomatic;
    uint32_t externalTrigger;
    uint32_t disableMask;
    uint32_t useNim1;
    uint32_t enableBLT;
    uint32_t hitSleep;
    uint32_t threshold[N_THRESHOLD];
    uint32_t disableSeparator;

    void print(std::ostream & os = std::cout) 
    {
      os << "WUT Configuration" << std::endl;
      ConfigurationBase::print(os);
      os << "  BridgeType             " << bridgeType       << std::endl;
      os << "  Link                   " << link             << std::endl;
      os << "  BoardNumber            " << boardNumber      << std::endl;
      os << "  EnableReadout          " << enableReadout    << std::endl;
      os << "  DebugLevel             " << debugLevel       << std::endl;
      os << "  ReadoutPeriod          " << readoutPeriod    << std::endl;
      os << "  UseNim1                " << useNim1          << std::endl;
      os << "  CalibrationDelay       " << calibrationDelay << std::endl;
      os << "  EnableBLT              " << enableBLT        << std::endl;
      os << "  HitSleep               " << hitSleep         << std::endl;
      os << "  DisableSeparator       " << disableSeparator << std::endl;
      os << "  BaseAddress          0x" << std::hex << baseAddress <<
	" (" << std::dec << baseAddress << ")" << std::endl;
      os << "  DisableMask          0x" << std::hex << disableMask <<
	" (" << std::dec << disableMask << ")" << std::endl;
      os << "  ExternalTrigger        " << externalTrigger << std::endl;
      for ( uint32_t i=0; i<N_THRESHOLD; i++)
      {
	os << "  Threshold[" << i << "]   " << threshold[i] << std::endl;
      }
    }
};

#endif

