//
//  WUTDriver.h  (W.Badgett)
//
//  Class to configure and readout via an optical VME link
//  of Jinyuan's WUT  Wave Union TDC
//

#ifndef _WUTDriver_h
#define _WUTDriver_h

#include <stdio.h>
#include <fstream>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
extern "C"
{
#include "CAENVMElib.h"
}
#include "LariatConfiguration.h"
#include "WUTFragment.h"

class WUTDriver
{
  public:
  enum
  {
    FIRMWARE_ADDR        = 0x00,
    THRESHOLD_ADDR       = 0x04,
    PATTERN_ADDR         = 0x08,
    CSR_ADDR             = 0x0C,
    DISABLE_ADDR         = 0x10,
    CONTROL_M            = 0xFFFF,
    DATA0_ADDR           = 0x1000,
    DATA1_ADDR           = 0x1800,
    TCY_HEADER_M         = 0x0FFFFFFF,
    TCY_HEADER_S         = 4,
    TCY_HIT_M            = 0x0F000000,
    TCY_HIT_S            = 24,
    HIT_CHANNEL_M        = 0xF,
    HIT_CHANNEL_S        = 20,
    HIT_TIME_M           = 0x000FFFFF,
    HIT_TIME_S           = 0 ,
    COARSE_TIME_M        = 0xFFF00,
    COARSE_TIME_S        = 8,
    COARSE_TIME_PEDESTAL = 96,

    FINE_TIME_M          = 0xFF,
    FINE_TIME_S          = 0,

    THRESHOLD_M          = 0xFF,
    THRESHOLD1_S         = 8,
    THRESHOLD3_S         = 24,
    TDC_CHANNEL_S        = 0,
    TDC_CHANNEL_M        =        0x3,
    CHANGE_PAGE_M        =       0x10,
    FREEZE_STATUS_M      =       0x20,  // Obsolete
    DISABLE_SEPARATOR_M  = 0X20,
    DISABLE_FILL_M       =       0x40,
    TRIGGER_ENABLE_M     =      0X800,
    TRIGGER_NIM0_M       =      0x200,
    TRIGGER_VME_M        =      0x100,
    USE_NIM1_M           =     0x2000,
    CALIB_VME_M          =     0x1000,
    BUFFER_PAGE_M        = 0x80000000,
    BUFFER_PAGE_S        = 31,
    BUFFER_STORING_M     = 0x04000000,
    WORD_COUNT_M         = 0x03FF0000,
    WORD_COUNT_S         = 16,
    MAX_WORDS            = 512,
    CALIB_DELAY_M        = 0xC000,
    CALIB_DELAY_S        = 14,
    N_BUFFERS            = 2,
    MARKER_M             = 0xF0000000,
    MARKER_S             = 28,
    TIME_FRAME_MARKER    = 0xF0000000,
    HIT_MARKER           = 0x40000000
  };

  bool ok;
  LariatConfiguration * config;
  WUTConfiguration    * wut;
  int handle;
  uint32_t csrMask;
  uint32_t thresholdMask;

  uint32_t firmwareVersion;

  uint32_t thresholdRegister;
  uint32_t threshold1;
  uint32_t threshold3;

  uint32_t patternWord;

  uint32_t csr;
  uint32_t tdcChannel;
  bool     changePage;
  bool     freezeStatus;
  uint32_t disableMask;
  uint32_t wordCount;
  uint32_t bufferActive;
  bool     bufferStoring;
  bool     triggerNim0;
  bool     triggerEnable;
  bool     triggerVme;
  bool     calibAutomatic;
  uint32_t calibDelay;
  bool     useNim1;
  bool     calibVme;

  ~WUTDriver();
  WUTDriver(LariatConfiguration * config_,
	    WUTConfiguration    * wut_,
	    int                   handle_);
  void print(std::ostream & os = std::cout);
  void readEvent(std::ostream *dataFile, int pollCount);

  void readConfiguration();
  uint32_t getWordCount();

 protected:
  uint32_t readAndFlip();
  void decodeCSR();
  uint32_t *buffer[N_BUFFERS];
  CVAddressModifier am[MAX_WORDS];
  CVDataWidth dw[MAX_WORDS];
  uint32_t page[N_BUFFERS][MAX_WORDS];
  CVErrorCodes retcods[MAX_WORDS];
  uint32_t pageRead;
  uint32_t base;

 private:
  //   struct WUTFragment::WutFrameHeader header;
  //   struct WUTFragment::WutHit *hits;

};


#endif
