////////////////////////////////////////////////////////////////////////
// Class:       CAENWrapperDump
// Module Type: analyzer
// File:        CAENWrapperDump_module.cc
// Description: Prints out information about each event.
////////////////////////////////////////////////////////////////////////

#include "art/Framework/Core/EDAnalyzer.h"
#include "art/Framework/Core/ModuleMacros.h"
#include "art/Framework/Principal/Event.h"
#include "art/Framework/Principal/Handle.h"

#include "art/Utilities/Exception.h"
#include "lariat-artdaq/Overlays/CAENWrapperFragment.hh"
#include "ModuleUtils.hh"
#include "CAENFragment.h"
#include "artdaq-core/Data/Fragments.hh"

#include <algorithm>
#include <cassert>
#include <cmath>
#include <fstream>
#include <iomanip>
#include <vector>
#include <iostream>

namespace lariat {
  class CAENWrapperDump;
}

class lariat::CAENWrapperDump : public art::EDAnalyzer {
public:
  explicit CAENWrapperDump(fhicl::ParameterSet const & pset);
  virtual ~CAENWrapperDump() {}

  virtual void analyze(art::Event const & evt);

private:
  std::string raw_data_label_;
  //  uint32_t num_waveforms_to_show_;
  //  uint32_t num_adcs_to_show_per_waveform_;
};


lariat::CAENWrapperDump::CAENWrapperDump(fhicl::ParameterSet const & pset)
    : EDAnalyzer(pset),
      raw_data_label_(pset.get<std::string>("raw_data_label"))// ,
      //      num_waveforms_to_show_(pset.get<uint32_t>("num_waveforms_to_show", 0)),
      //      num_adcs_to_show_per_waveform_(pset.get<uint32_t>("num_adcs_to_show_per_waveform", 0))
{
}

void lariat::CAENWrapperDump::analyze(art::Event const & evt)
{

  art::EventNumber_t spillNumber = evt.event();

  art::Handle<artdaq::Fragments> raw;
  evt.getByLabel(raw_data_label_, "SPILL", raw);

  if ( raw.isValid() ) {

    if (raw->size() != 1 ) {
      throw cet::exception("Error in CAENWrapperDump module: expected one and only one spill fragment");
    }
    
    const auto& frag((*raw)[0]);

    std::vector<const uint8_t*> lariatFragPtrs;

    LariatFragment::LariatFragmentType lariatFragType = LariatFragment::LariatFragmentType::FRAGMENT_TYPE_CAEN_ADC;

    getLariatFragments(frag, 
     		       lariatFragType,
		       lariatFragPtrs
		       );

    std::cout << std::endl;
    std::cout << "Run " << evt.run() << ", subrun " << evt.subRun()
       	      << ", spill " << spillNumber << " has " << lariatFragPtrs.size()
   	      << " fragment(s) of type " << lariatFragType << std::endl;
      
    for (size_t idx = 0; idx < lariatFragPtrs.size(); ++idx) {

      lariat::CAENWrapperFragment bb( lariatFragPtrs[idx]  );

      CAENFragment frag(*bb.GetFragment());

      frag.print();

      std::cout << std::endl;
      std::cout << "######################################################################" << std::endl;
      std::cout << std::endl;
      std::cout << std::endl;
    }
  } else {
    throw cet::exception("Error in CAENWrapperDump_module: expected one and only one spill fragment");
  }

  std::cout << std::endl;

}

DEFINE_ART_MODULE(lariat::CAENWrapperDump)
