////////////////////////////////////////////////////////////////////////
// Class:       Summary
// Module Type: analyzer
// File:        Summary_module.cc
//
// Generated at Wed Oct 22 11:47:26 2014 by John Freeman using artmod
// from cetpkgsupport v1_07_00.
////////////////////////////////////////////////////////////////////////

// JCF, 10/24/14

// "ExtractBinary" is a simple utility Art module which will take the
// entire payload of the artdaq::Fragment object and save it to disk
// (using the filename "output_filename") as a binary file. The main
// purpose for this module is to be able to cross check that the
// payload is the same as a pre-existing binary which was processed by
// the SpillFragmentReader fragment generator (modulo the 8-byte-wide
// padding at the end of the payload expected in artdaq v1_06_02)

#include "art/Framework/Core/EDAnalyzer.h"
#include "art/Framework/Core/ModuleMacros.h"
#include "art/Framework/Principal/Event.h"
#include "art/Framework/Principal/Handle.h"
#include "art/Framework/Principal/Run.h"
#include "art/Framework/Principal/SubRun.h"
#include "art/Utilities/InputTag.h"
#include "fhiclcpp/ParameterSet.h"
#include "messagefacility/MessageLogger/MessageLogger.h"

#include "artdaq-core/Data/Fragments.hh"
#include "ModuleUtils.hh"

#include <vector>

namespace lariat {
  class Summary;
}

class lariat::Summary : public art::EDAnalyzer {
public:
  explicit Summary(fhicl::ParameterSet const & p);
  // The destructor generated by the compiler is fine for classes
  // without bare pointers or other resource use.

  // Plugins should not be copied or assigned.
  Summary(Summary const &) = delete;
  Summary(Summary &&) = delete;
  Summary & operator = (Summary const &) = delete;
  Summary & operator = (Summary &&) = delete;

  // Required functions.
  void analyze(art::Event const & e) override;


private:

  std::string output_filename_;

};


lariat::Summary::Summary(fhicl::ParameterSet const & ps)
  :
  EDAnalyzer(ps),
  output_filename_(ps.get<std::string>("output_filename", "ExtractBinary.dat"))
{}

void lariat::Summary::analyze(art::Event const & evt)
{

  art::Handle<artdaq::Fragments> raw;

  evt.getByLabel("daq", "SPILL", raw);

  if ( !raw.isValid() || raw->size() != 1) {

    throw cet::exception("Error in ExtractBinary_module: expected one and only one spill fragment");

  } else {

    const auto& spillfrag((*raw)[0]);

    std::ofstream binaryFile(output_filename_);

    binaryFile.write( reinterpret_cast<const char*>( &*spillfrag.dataBegin() ), spillfrag.dataSize()*sizeof(artdaq::RawDataType) );
  }
}


DEFINE_ART_MODULE(lariat::Summary)
