
#include "ModuleUtils.hh"

namespace lariat {

void getLariatFragments(const artdaq::Fragment& artdaqFrag, 
			LariatFragment::LariatFragmentType desiredFragType,
			std::vector<const uint8_t*>& lariatPtrs
			) {

  if (artdaqFrag.dataBegin() == artdaqFrag.dataEnd() ) {
    throw cet::exception("Error in ModuleUtils: artdaq::Fragment appears to contain no payload");
  }

  // JCF, 5/6/14 

  // We'll need to use raw pointers rather than smart pointers,
  // otherwise a crash will occur since other resources are being used to
  // control the memory in the artdaq::Fragment

  const uint8_t* bytePtr = 
    reinterpret_cast<const uint8_t*>( &*artdaqFrag.dataBegin() );
  
  const uint8_t* endPtr = reinterpret_cast< const uint8_t* >( &*artdaqFrag.dataEnd() );

  // JCF 5/8/14: Subtract off the size of an artdaq::Fragment's
  // dataType (8 bytes), so that junk isn't read in past the end of
  // the last lariat fragment...

  while ( bytePtr < endPtr - sizeof(artdaq::RawDataType) ) {

    const uint32_t* wordPtr = reinterpret_cast<const uint32_t*>( bytePtr );

    uint32_t fragSize = *wordPtr;
    uint32_t fragType = *(wordPtr+1);

    if ( fragType == static_cast<uint32_t>( desiredFragType ) ) {
      lariatPtrs.emplace_back( bytePtr );
    }

    bytePtr += fragSize;

    //    std::cout << "Fragsize is " << fragSize << ", fragtype is " << fragType << ", bytePtr now at (decimal) " << reinterpret_cast<long>( bytePtr ) << std::endl;
  }

}

}
