////////////////////////////////////////////////////////////////////////
// Class:       PawelDump
// Module Type: analyzer
// File:        PawelDump_module.cc
// Description: Prints out information about each event.
////////////////////////////////////////////////////////////////////////
/*
-fragment/board/spill 
-fragment id corresponds to v1740
- . !!:$ previous command and last token in previous
-lariarReset 0 1


*/
#include "art/Framework/Core/EDAnalyzer.h"
#include "art/Framework/Core/ModuleMacros.h"
#include "art/Framework/Principal/Event.h"
#include "art/Framework/Principal/Handle.h"

#include "art/Framework/Principal/Run.h"
#include "art/Utilities/InputTag.h"
//#include "art/Framework/Core/ModuleMacros.h"
//#include "art/Framework/Services/Optional/TFileService.h"

#include "ModuleUtils.hh"

#include "WUTFragment.h"
#include "LariatFragment.h"
#include "TDCFragment.h"

#include "artdaq-core/Data/Fragments.hh"
#include "art/Utilities/Exception.h"

#include "lariat-artdaq/Overlays/WUTHitFragment.hh"
#include "lariat-artdaq/Overlays/TDCEventFragment.hh"
#include "lariat-artdaq/Overlays/TDCSpillFragment.hh"
#include "lariat-artdaq/Overlays/TDCControllerFragment.hh"
#include "lariat-artdaq/Overlays/FragmentType.hh"
#include "lariat-artdaq/Overlays/CAENADCFragment.hh"

#include "cetlib/exception.h"

#include "TRootCanvas.h"
#include "TCanvas.h"
#include "TGraph.h"
#include "TAxis.h"
#include "TH1D.h"
#include "TH2D.h"
#include "TStyle.h"
#include "TFile.h"
#include "TMath.h"




#include <cassert>
#include <cmath>
#include <fstream>
#include <iomanip>
#include <numeric>
#include <vector>
#include <functional>
#include <algorithm>
#include <iostream>
#include <sstream>
#include <initializer_list>
#include <limits>
#include <unordered_set>

#include <cstdlib>
#include <stdint.h>
using std::cout;
using std::cerr;
using std::endl;
namespace lariat {
  //class PawelDump;


class PawelDump : public art::EDAnalyzer {
public:
  explicit PawelDump(fhicl::ParameterSet const & pset);
  virtual ~PawelDump();

  virtual void analyze(art::Event const & e);

private:

std::unique_ptr<TCanvas> canvas_[2];
    std::vector<Double_t> x_;
    int prescale_;
    bool digital_sum_only_;
    art::RunNumber_t current_run_;
    std::size_t nSamples_;
    std::size_t nChannels_;
	//MODIFIED
    int samples_per_channel=1536; 
    int board_channels=64;
    int samples_ct=0;
    int channels_ct=0;
	int xbin=1;
	int ybin=1;
	double sum_adc=0.;
	int pedestal_ct=1000;
	double pedestal_mean=0;
	double pedestal_std=0.;
	int hits=0;
	int ecounter=0;
	int ind2=0;
	bool savexytdc=false;
	int TDCnum=0;
    std::size_t num_x_plots_;
    std::size_t num_y_plots_;
    std::size_t totalEventsSeen;
    std::unordered_set<uint32_t> eventsSeenInSpill;
    std::vector<std::vector<int>> tdchits;
std::vector<int>tdchit;
    std::vector<std::vector<std::vector<int>>> tdchits2;
//std::vector<int>tdchit2;
    std::vector<std::string> fragment_type_labels_;
    std::vector<artdaq::Fragment::fragment_id_t> fragment_ids_;

    std::string histos_filename_;
 //art::ServiceHandle<art::TFileService> tfs_;
    std::vector<std::shared_ptr<TGraph>> graphs_;
    std::vector<std::shared_ptr<TH1D>> histograms_;
    std::vector<std::shared_ptr<TH2D>> twoDhistograms_;
  std::vector<std::shared_ptr<TH1D>> wvform_;
  std::vector<std::shared_ptr<TH2D>> histograms_av_adc_;
  std::vector<std::shared_ptr<TH2D>> histograms_hits_;
  std::vector<std::shared_ptr<TH1D>> histograms_timestamp_;
  std::vector<std::shared_ptr<TH1D>> histograms2_;
  std::vector<std::shared_ptr<TH1D>> pedestal_;
  std::vector<std::shared_ptr<TH2D>> pedestal_means_;
  std::vector<std::shared_ptr<TH2D>> pedestal_rms_;
  std::vector<std::shared_ptr<TH2D>> pedestal_mean_lastn_;
  std::vector<std::shared_ptr<TH2D>> pedestal_rms_lastn_;
  std::vector<std::shared_ptr<TH2D>> pedestal_mean_last_;
  std::vector<std::shared_ptr<TH2D>> pedestal_rms_last_;
  std::vector<std::shared_ptr<TH2D>> tdc_xy_;
  std::vector<std::shared_ptr<TH2D>> tdc_timestamp_;
  std::vector<std::shared_ptr<TH1D>> tdc_x1_;
  std::vector<std::shared_ptr<TH1D>> tdc_y1_;
  std::vector<std::shared_ptr<TH2D>> hits_x_in;
    std::map<artdaq::Fragment::fragment_id_t, std::size_t> id_to_index_;
  std::string raw_data_label_;
  std::string event_frag_type_;
  uint32_t max_wuts_to_show_;
//TDC
  
  std::string controller_frag_type_;
  std::string spill_frag_type_;
 
  uint32_t max_tdcs_to_show_;

//TDC
};

}
lariat::PawelDump::PawelDump (fhicl::ParameterSet const & ps):
 art::EDAnalyzer(ps),
  prescale_(ps.get<int> ("prescale")), 
  digital_sum_only_(ps.get<bool> ("digital_sum_only", false)), 
  current_run_(0), 
  nSamples_(1536),
  nChannels_(64),
  num_x_plots_(ps.get<std::size_t>("num_x_plots", std::numeric_limits<std::size_t>::max() )),
  num_y_plots_(ps.get<std::size_t>("num_y_plots", std::numeric_limits<std::size_t>::max() )),
  totalEventsSeen(0),
  tdchits2(8),
  fragment_type_labels_(ps.get<std::vector<std::string>>("fragment_type_labels")),
  fragment_ids_(ps.get<std::vector<artdaq::Fragment::fragment_id_t> >("fragment_ids")),
  histos_filename_(ps.get<std::string>("histos_filename","renameme_histos.root")),
  graphs_( fragment_ids_.size() ), 
  histograms_( fragment_ids_.size() ),
  twoDhistograms_( fragment_ids_.size() ),
  wvform_( fragment_ids_.size()*board_channels ),
  histograms_av_adc_( fragment_ids_.size()),
  histograms_hits_( fragment_ids_.size()),
  histograms_timestamp_( fragment_ids_.size()),
  histograms2_( fragment_ids_.size()*board_channels),
  pedestal_( fragment_ids_.size()*board_channels ),
  pedestal_means_( fragment_ids_.size()*board_channels ),
  pedestal_rms_( fragment_ids_.size()*board_channels ),
  tdc_xy_( fragment_ids_.size()*board_channels ),
  tdc_timestamp_( fragment_ids_.size()*board_channels ),
  tdc_x1_( fragment_ids_.size()*board_channels),
  tdc_y1_( fragment_ids_.size()*board_channels ),
  raw_data_label_(ps.get<std::string>("raw_data_label")),
      //event_frag_type_(ps.get<std::string>("event_frag_type")),
      max_wuts_to_show_(ps.get<uint32_t>("max_wuts_to_show", 0)),
 // raw_data_label_(ps.get<std::string>("raw_data_label")),
     // controller_frag_type_(ps.get<std::string>("controller_frag_type")),
      //spill_frag_type_(ps.get<std::string>("spill_frag_type")),
  //    event_frag_type_(ps.get<std::string>("event_frag_type")),
      max_tdcs_to_show_(ps.get<uint32_t>("max_tdcs_to_show", 0))
		      
{

   for (std::size_t i_f = 0; i_f < fragment_ids_.size(); ++i_f) {
     id_to_index_[ fragment_ids_[i_f] ] = i_f;
   }

 sort( fragment_type_labels_.begin(), fragment_type_labels_.end() );
  fragment_type_labels_.erase( unique( fragment_type_labels_.begin(), fragment_type_labels_.end() ), fragment_type_labels_.end() );
}

lariat::PawelDump::~PawelDump()
{


}

void lariat::PawelDump::analyze(art::Event const & e)
{
std::cout<<"historgam plotting module "<<std::endl;

e.event();
  static std::size_t evt_cntr = -1;
  evt_cntr++;
  art::EventNumber_t spillNumber = e.event();


  std::size_t max_adc_count = pow(2, 12); // 12 bits for a V1740 
  std::size_t total_adc_values = nSamples_ * nChannels_;

for(int j=0;j<4;j++){

tdc_timestamp_[j].reset( new TH2D(Form("tdctimestamp_%d_plane_%d", spillNumber, j),"",     1500, -0.5, 1500 - 0.5, 20,-0.5, 20 - 0.5 ) );
      tdc_timestamp_[j]->SetTitle (Form ("Frag %d, plane %d",spillNumber, j) );
	
      tdc_timestamp_[j]->GetXaxis()->SetTitle( "x #" );
      tdc_timestamp_[j]->GetYaxis()->SetTitle( "y #" );

      tdc_timestamp_[j]->SetMinimum(0);
      tdc_timestamp_[j]->SetMaximum( max_adc_count );

      tdc_timestamp_[j]->SetStats(kFALSE);

}

  art::Handle<artdaq::Fragments> raw;

  e.getByLabel("daq", "SPILL", raw);

  if ( !raw.isValid() || raw->size() != 1) {

    throw cet::exception("Error in WFViewer_module: expected one and only one spill fragment");

  } else {
    TFile f(histos_filename_.c_str(), "UPDATE");
    f.cd();
    eventsSeenInSpill.clear();
    const auto& spillfrag((*raw)[0]);

    std::vector<const uint8_t*> lariatFragPtrs;

    const LariatFragment::LariatFragmentType lariatFragType = 
      LariatFragment::LariatFragmentType::FRAGMENT_TYPE_CAEN_ADC ;

    getLariatFragments(spillfrag,
                       lariatFragType,
                       lariatFragPtrs
                       );

    std::cout << std::endl;
    std::cout << "Run " << e.run() << ", subrun " << e.subRun()
              << ", spill " << spillNumber << " has " << lariatFragPtrs.size()
              << " fragment(s) of type CAEN ADC " << lariatFragType << std::endl;

    if ( lariatFragPtrs.size() % fragment_ids_.size() != 0) {
      throw cet::exception("# of CAENADC fragments found is not a multiple of the number of fragment_ids supplied");
    } else {
      std::cout << "There appear to be " << lariatFragPtrs.size() / fragment_ids_.size() <<
	" triggered events in the spill" << std::endl;
    }

    // Plot the integrated distribution of ADC counts for each
    // fragment_id. Also, for every Nth triggered event, where N is
    // the "prescale" setting, if "digital_sum_only" is set to false
    // in the FHiCL string, then plot, for the Nth event, a graph of
    // the ADC values across all channels in each board_id /
    // fragment_id combo

    for (auto fragPtr : lariatFragPtrs) {
    
      const CAENADCFragment* caenPtr = new CAENADCFragment(reinterpret_cast<const uint8_t*>(fragPtr ));

      uint32_t eventCounter = caenPtr->getHeader()->eventCounter;
ecounter=int(eventCounter);
      auto got = eventsSeenInSpill.find( eventCounter );

      if ( got == eventsSeenInSpill.end() ) {
	eventsSeenInSpill.emplace( eventCounter );
	totalEventsSeen++;
      }


      if ( caenPtr->total_values() != total_adc_values ) {
	throw cet::exception("Error in WFViewer: unexpected number of ADC values found in fragment");
      }

      artdaq::Fragment::fragment_id_t fragment_id = caenPtr->getHeader()->boardId;
      std::size_t ind = id_to_index_[ fragment_id ];
	ind2=int(ind);
std::cout<<"index  "<<ind<<" fragment id "<<fragment_id<<std::endl;
    	    tdc_timestamp_[0]->Fill(int(double(caenPtr->getHeader()->timeStampSec)/1000000.), int(caenPtr->getHeader()->boardId ) );
	std::cout<<int(double(caenPtr->getHeader()->timeStampSec)/1000000.)<<" CAEN timestamp/10^6 "<<int(caenPtr->getHeader()->boardId ) <<" CAEN number "<<std::endl;
      // If a histogram doesn't exist for this fragment_id , create it

      if (!histograms_[ind]) {

	//histograms_[ind] = std::shared_ptr<TH1D>( 
		//				 tfs_->make<TH1D>( Form ("Fragment_%d_hist", fragment_id), 
			//					   "", max_adc_count+1, -0.5, 
				//				   max_adc_count + 0.5 ) );
	histograms_[ind].reset( new TH1D(Form ("Fragment_%d_hist", fragment_id), 
					 "", max_adc_count+1, -0.5, 
					 max_adc_count + 0.5 ) );
	histograms_[ind]->SetTitle (Form ("Frag %d, Type CAENADC", fragment_id ) );
	histograms_[ind]->GetXaxis()->SetTitle("ADC value");
      }
     	if(! histograms_av_adc_[ind]){ 
	//histograms_av_adc_[ind] = std::shared_ptr<TH2D>(tfs_->make<TH2D>( Form ("Fragment_%d_av_adc", fragment_id), "", fragment_ids_.size(), 0, fragment_ids_.size(),fragment_ids_.size(),0,fragment_ids_.size()));
	histograms_av_adc_[ind].reset( new TH2D(Form ("Fragment_%d_av_adc", fragment_id), "", fragment_ids_.size(), 0, fragment_ids_.size(),fragment_ids_.size(),0,fragment_ids_.size()) );


     	 histograms_av_adc_[ind]->SetTitle (Form ("Frag %d , Type CAENADC", fragment_id )  );
     	 histograms_av_adc_[ind]->GetXaxis()->SetTitle("Average ADC values");
	}

	if(! histograms_hits_[ind]){ 
	//histograms_hits_[ind] = std::shared_ptr<TH2D>(tfs_->make<TH2D>( Form ("Fragment_%d_hits", fragment_id), "", fragment_ids_.size(), 0, fragment_ids_.size(),fragment_ids_.size(),0,fragment_ids_.size()));
	histograms_hits_[ind].reset( new TH2D(Form ("hits_%d_av_adc", fragment_id), "", fragment_ids_.size(), 0, fragment_ids_.size(),fragment_ids_.size(),0,fragment_ids_.size()) );


     	 histograms_hits_[ind]->SetTitle (Form ("hits Frag %d", fragment_id ) );
     	 histograms_hits_[ind]->GetXaxis()->SetTitle("Hits in channels");
	}


//for each channel

for(int ch=0;ch<board_channels;ch++){

     	if (!histograms2_[ind*board_channels+ch]) {
	// histograms2_[ind*board_channels+ch] = std::shared_ptr<TH1D>(tfs_->make<TH1D>( Form ("histograms2_%d_channel%d_hist", fragment_id, ch), "", samples_per_channel, 0., samples_per_channel ));
	histograms2_[ind*board_channels+ch].reset( new TH1D(Form ("histograms2_%d_channel%d_hist", fragment_id, ch), "", samples_per_channel, 0., samples_per_channel ) );


     	histograms2_[ind*board_channels+ch]->SetTitle (Form ("Frag %d, Channel No %d", fragment_id, ch ) );
     	 histograms2_[ind*board_channels+ch]->GetXaxis()->SetTitle("ADC value");
	}
     	if (!wvform_[ind*board_channels+ch]) { 
	//wvform_[ind*board_channels+ch] = std::shared_ptr<TH1D>(tfs_->make<TH1D>( Form ("wvform_%d_channel%d_hist", fragment_id, ch), "", samples_per_channel, 0., samples_per_channel ));
	wvform_[ind*board_channels+ch].reset( new TH1D(Form ("wvform_%d_channel%d_hist", fragment_id, ch), "", samples_per_channel, 0., samples_per_channel ) );


     	 wvform_[ind*board_channels+ch]->SetTitle (Form ("Frag %d, Channel No %d", fragment_id, ch ) );
     	 wvform_[ind*board_channels+ch]->GetXaxis()->SetTitle("ADC value");
	}
     	 	if (!pedestal_[ind*board_channels+ch]){
		//pedestal_[ind*board_channels+ch] = std::shared_ptr<TH1D>(tfs_->make<TH1D>( Form ("pedestal_%d_channel%d_hist", fragment_id, ch), "", 2200, 0., 2200 ));

	pedestal_[ind*board_channels+ch].reset( new TH1D(Form ("pedestal_%d_channel%d_hist", fragment_id, ch), "", 2200, 0., 2200 ) );


     	 pedestal_[ind*board_channels+ch]->SetTitle (Form ("Frag %d, Channel No %d", fragment_id, ch ) );
     	 pedestal_[ind*board_channels+ch]->GetXaxis()->SetTitle("ADC value");
	
		}



  
		
	}
std::cout<<"created histograms"<<std::endl;
      // For every event, fill the histogram (prescale is ignored here)
  	channels_ct=0;
	samples_ct=0;
	sum_adc=0.;
	xbin=1;
	ybin=1;
	pedestal_mean=0.;
	pedestal_std=0.;

      for (auto val = caenPtr->dataBegin(); val != caenPtr->dataEnd(); ++val ){

	histograms_[ind]->Fill( *val );
	
		

		}//data loop

			//histograms_av_adc_[ind]->Write(0,TObject::kOverwrite);
			//histograms_timestamp_[ind]->Write(0,TObject::kOverwrite);
			//histograms_hits_[ind]->Write(0,TObject::kOverwrite);
//std::cout<<"wrote histograms"<<std::endl;
     // break;
//std::cout<<"breaking"<<std::endl;
      if (totalEventsSeen % prescale_ - 1 != 0 && prescale_ > 1) {
	continue;
      }

      // If we pass the prescale, then if we're not going with
      // digital_sum_only, plot the ADC counts for this particular event/fragment_id


    // And now prepare the graphics without actually drawing anything yet
      //twoDhistograms_[ind] = std::shared_ptr<TH2D>( tfs_->make<TH2D>( Form("Fragment_%d_spill_%d_eventCounter_%d_twoDhist", fragment_id, spillNumber, eventCounter), "",
	      twoDhistograms_[ind].reset( new TH2D(Form("Fragment_%d_spill_%d_eventCounter_%d_twoDhist", fragment_id, spillNumber, eventCounter), "",
					   nSamples_, -0.5, 
					   nSamples_ - 0.5, nChannels_,
					   -0.5, nChannels_ - 0.5 ) );							

      twoDhistograms_[ind]->SetTitle (Form ("Frag %d, Spill %d, Event %d", fragment_id, 
					    spillNumber, eventCounter) );
	
      twoDhistograms_[ind]->GetXaxis()->SetTitle( "Sample #" );
      twoDhistograms_[ind]->GetYaxis()->SetTitle( "Channel #" );

      twoDhistograms_[ind]->SetMinimum(0);
      twoDhistograms_[ind]->SetMaximum( max_adc_count );

      twoDhistograms_[ind]->SetStats(kFALSE);

//pedestal means
    //  pedestal_means_[ind] = std::shared_ptr<TH2D>( tfs_->make<TH2D>( Form("Pedestal_mean_%d_spill_%d_eventCounter_%d_", fragment_id, spillNumber, eventCounter), "",
								    //  8, -0.5, 
								      //8 - 0.5, 8,
								      //-0.5, 8 - 0.5 ) );
								      
pedestal_means_[ind].reset( new TH2D(Form("Pedestal_mean_%d_spill_%d_eventCounter_%d_", fragment_id, spillNumber, eventCounter), "",
								      8, -0.5, 
								      8 - 0.5, 8,
								      -0.5, 8 - 0.5 ) );

      pedestal_means_[ind]->SetTitle (Form ("Frag %d, Spill %d, Event %d", fragment_id, 
					    spillNumber, eventCounter) );
	
      pedestal_means_[ind]->GetXaxis()->SetTitle( "Channel #" );
      pedestal_means_[ind]->GetYaxis()->SetTitle( "Channel #" );

     // pedestal_means_[ind]->SetMinimum(0);
     // pedestal_means_[ind]->SetMaximum( max_adc_count );

      pedestal_means_[ind]->SetStats(kFALSE);
//pedestal rms
     // pedestal_rms_[ind] = std::shared_ptr<TH2D>( tfs_->make<TH2D>( //Form("Pedestal_rms_%d_spill_%d_eventCounter_%d_", fragment_id, spillNumber, eventCounter), "",
								  //    8, -0.5, 
								    //  8 - 0.5, 8,
								      //-0.5, 8 - 0.5 ) );
pedestal_rms_[ind].reset( new TH2D(Form("Pedestal_rms_%d_spill_%d_eventCounter_%d_", fragment_id, spillNumber, eventCounter), "",
								      8, -0.5, 
								      8 - 0.5, 8,
								      -0.5, 8 - 0.5 ) );
      pedestal_rms_[ind]->SetTitle (Form ("Frag %d, Spill %d, Event %d", fragment_id, 
					    spillNumber, eventCounter) );
	
      pedestal_rms_[ind]->GetXaxis()->SetTitle( "Channel #" );
      pedestal_rms_[ind]->GetYaxis()->SetTitle( "Channel #" );

     // pedestal_means_[ind]->SetMinimum(0);
     // pedestal_means_[ind]->SetMaximum( max_adc_count );

      pedestal_rms_[ind]->SetStats(kFALSE);
      for (uint32_t i_x = 0; i_x < nSamples_; ++i_x) {
	for (uint32_t i_y = 0; i_y < nChannels_; ++i_y) {
			   		if(100< i_x && int(i_x)<pedestal_ct){
			pedestal_[ind*board_channels+i_y]->Fill( *( caenPtr->dataBegin() + i_y*nSamples_ + i_x  ) );
			//std::cout<<*( caenPtr->dataBegin() + i_y*nSamples_ + i_x  )<<std::endl;
				} 
	  twoDhistograms_[ind]->SetBinContent( i_x+1, i_y+1, *( caenPtr->dataBegin() + i_y*nSamples_ + i_x  ) );
	}
      }
uint32_t xb = 0;
uint32_t yb = 0;
for (uint32_t i_y = 0; i_y < nChannels_; ++i_y) {
	   	
		//	std::cout<<"mean value "<<pedestal_[ind*board_channels+i_y]->GetMean()<<std::endl;
		//	std::cout<<"rms value "<<pedestal_[ind*board_channels+i_y]->GetRMS()<<std::endl;
			 pedestal_means_[ind]->SetBinContent( xb+1,yb+1, pedestal_[ind*board_channels+i_y]->GetMean() );
			 pedestal_rms_[ind]->SetBinContent( xb+1,yb+1, pedestal_[ind*board_channels+i_y]->GetRMS() );
			xb++;
			if(xb>7){

			 yb++;
			xb=0;
			}
	  
	}
//writing histograms
      twoDhistograms_[ind]->Draw("COLZ");
      twoDhistograms_[ind]->Write();

//writing histograms

      pedestal_means_[ind]->Write();
      pedestal_rms_[ind]->Write();
     // pedestal_[ind]->Write();      




    // Draw the histogram



    // And, if desired, the Nth event's ADC counts
  


    }
//std::cout<<"index "<<ind2<<std::endl;
    f.Close();

  }
//end of CAEN

//TDC




  if ( ! raw.isValid() || raw->size() != 1 ) {
    throw cet::exception("Error in TDCDump module: expected one and only one spill fragment");
  } else {
        TFile f2(histos_filename_.c_str(), "UPDATE");
    f2.cd();
    const auto& spillfragt((*raw)[0]);
    
    std::vector<LariatFragment::LariatFragmentType> lariatFragTypest;

    lariatFragTypest.push_back( LariatFragment::LariatFragmentType::FRAGMENT_TYPE_TDC_CONTROLLER_HEADER );
    lariatFragTypest.push_back( LariatFragment::LariatFragmentType::FRAGMENT_TYPE_TDC_TDC_SPILL_HEADER );
    lariatFragTypest.push_back( LariatFragment::LariatFragmentType::FRAGMENT_TYPE_TDC_TDC_EVENT_HEADER );
      //tdc_xy_[ind2] = std::shared_ptr<TH2D>( tfs_->make<TH2D>( Form("tdc_%d_spill_%d_eventCounter_%d_xy", ind2, spillNumber, ecounter), "",
	//							      128, -0.5, 
		//						      128 - 0.5, 128,
			//					      -0.5, 128 - 0.5 ) );

    for ( auto lariatFragType : lariatFragTypest ) {

      std::vector<const uint8_t*> lariatFragPtrst;

      getLariatFragments(spillfragt, 
			 lariatFragType,
			 lariatFragPtrst
			 );

      std::cout << std::endl;
      std::cout << "PAWEL DUMP TDC Run " << e.run() << ", subrun " << e.subRun()
		<< ", spill " << spillNumber << " has " << lariatFragPtrst.size()
		<< " fragment(s) of type " << lariatFragType << std::endl;
    //    tdc_xy_( int(lariatFragPtrst.size()));
  	//tdc_x1_(int(lariatFragPtrst.size()) );
  	//tdc_y1_(int(lariatFragPtrst.size()));
int planestdc=4;
for(int j=0;j<planestdc;j++){
tdc_xy_[j].reset( new TH2D(Form("tdcXY_%d_plane_%d", spillNumber, j),"",     128,-0.5,	      128 - 0.5, 128, -0.5, 128 - 0.5 ) );
      tdc_xy_[j]->SetTitle (Form ("Frag %d, plane %d",spillNumber, j) );
	
      tdc_xy_[j]->GetXaxis()->SetTitle( "y #" );
      tdc_xy_[j]->GetYaxis()->SetTitle( "y #" );

      tdc_xy_[j]->SetMinimum(0);
      tdc_xy_[j]->SetMaximum( max_adc_count );

      tdc_xy_[j]->SetStats(kFALSE);



}
      for (size_t idx = 0; idx < lariatFragPtrst.size(); ++idx) {

ind2=idx;
std::cout<<"index "<<idx<<std::endl;

//x1
     // tdc_x1_[ind2] = std::shared_ptr<TH1D>( tfs_->make<TH1D>( Form("tdc_%d_spill_%d_eventCounter_%d_x1", ind2, spillNumber, ecounter), "",
	//							      128, -0.5, 
//								      128 - 0.5) );
tdc_x1_[ind2].reset( new TH1D(Form("tdcx1_%d_spill_%d_x1", ind2, spillNumber), "",
								      128, -0.5, 
								      128 - 0.5) );
      tdc_x1_[ind2]->SetTitle (Form ("TDC x Frag %d, Spill %d", ind2, 
					    spillNumber) );
	
      tdc_x1_[ind2]->GetXaxis()->SetTitle( "x #" );

      tdc_x1_[ind2]->SetStats(kFALSE);

//y1
tdc_y1_[ind2].reset( new TH1D(Form("tdcy1_%d_spill_%d", ind2, spillNumber), "",
								      128, -0.5, 
								      128 - 0.5) );

      tdc_y1_[ind2]->SetTitle (Form ("TDC y Frag %d, Spill %d", ind2, 
					    spillNumber) );
	
      tdc_y1_[ind2]->GetXaxis()->SetTitle( "y #" );

      tdc_y1_[ind2]->SetStats(kFALSE);
	switch ( lariatFragType ) {
	case LariatFragment::LariatFragmentType::FRAGMENT_TYPE_TDC_CONTROLLER_HEADER:
	  {
	    lariat::TDCControllerFragment tdcc( lariatFragPtrst[idx]  );

	    tdcc.printHeader();
	    break;
	  }
	case LariatFragment::LariatFragmentType::FRAGMENT_TYPE_TDC_TDC_SPILL_HEADER:
	  {
	    lariat::TDCSpillFragment tdcs( lariatFragPtrst[idx]  );

	    tdcs.printHeader();

		std::cout<<"number of tdc!!!!! "<<TDCnum<<std::endl;
	    break;
	  }
	case LariatFragment::LariatFragmentType::FRAGMENT_TYPE_TDC_TDC_EVENT_HEADER:
	  {
	    lariat::TDCEventFragment tdce( lariatFragPtrst[idx]  );

	    tdce.printHeader();
	    tdc_timestamp_[0]->Fill(int(double(tdce.TDCTimestamp())/1000000.), int(tdce.TDCNumber()) );
	std::cout<<int(double(tdce.TDCTimestamp())/1000000.)<<" timestamp/10^6 "<<int(tdce.TDCNumber())<<" tdc number "<<std::endl;
		TDCnum=int(tdce.TDCNumber());
	    if (max_tdcs_to_show_ > 0) {

	      uint32_t actual_to_show = max_tdcs_to_show_ <= tdce.total_values() ?
		 	  max_tdcs_to_show_ : tdce.total_values() ;

	      std::cout << std::endl;
	      std::cout << "First " << actual_to_show
			<< " TDC values in the fragment: " 
			<< std::endl;
		
	      for (uint32_t i_tdc = 0; i_tdc < tdce.total_values(); ++i_tdc) {
		//std::cout <<"tdc number "<<TDCnum<< " total "<<tdce.total_values()<<" (Channel " << static_cast<int>( (*(tdce.dataBegin() + i_tdc)).channel ) <<  ", Timebin " << (*(tdce.dataBegin() + i_tdc)).timeBin << ") ";
		tdchit.clear();
		tdchit.push_back(TDCnum);
		tdchit.push_back(static_cast<int>( (*(tdce.dataBegin() + i_tdc)).timeBin ));
		tdchit.push_back(static_cast<int>( (*(tdce.dataBegin() + i_tdc)).channel ));
		tdchit.push_back(tdce.TDCHits());
		tdchits.push_back(tdchit);
		//first plane x
		if(TDCnum==1 || TDCnum==2 ){
		tdchits2[0].push_back(tdchit);
		std::cout<<" hits-x "<<tdchits2[0].size()<<std::endl;
			}
		//first plane y
		if(TDCnum==3 || TDCnum==4){
		tdchits2[1].push_back(tdchit);
		std::cout<<" hits-y "<<tdchits2[1].size()<<std::endl;
			}
		if(TDCnum==16) savexytdc=true;
		     /*   switch(TDCnum){
        // first chamber
        case 1:
          tdc_x1_[ind2]->Fill(static_cast<int>( (*(tdce.dataBegin() + i_tdc)).channel ));
          break;
        case 2:
          tdc_x1_[ind2]->Fill(64+static_cast<int>( (*(tdce.dataBegin() + i_tdc)).channel) );
          break;
        case 3:
          tdc_y1_[ind2]->Fill(static_cast<int>( (*(tdce.dataBegin() + i_tdc)).channel ));
          break;
        case 4:
          tdc_y1_[ind2]->Fill(64 + static_cast<int>( (*(tdce.dataBegin() + i_tdc)).channel ));
          break;
        // second chamber
        case 5:
          tdc_x1_[ind2]->Fill(static_cast<int>( (*(tdce.dataBegin() + i_tdc)).channel ));
          break;
        case 6:
          tdc_x1_[ind2]->Fill(64+static_cast<int>( (*(tdce.dataBegin() + i_tdc)).channel) );
          break;
        case 7:
          tdc_y1_[ind2]->Fill(static_cast<int>( (*(tdce.dataBegin() + i_tdc)).channel ));
          break;
        case 8:
          tdc_y1_[ind2]->Fill(64 + static_cast<int>( (*(tdce.dataBegin() + i_tdc)).channel ));
          break;
        case 9:
          tdc_x1_[ind2]->Fill(static_cast<int>( (*(tdce.dataBegin() + i_tdc)).channel ));
          break;
        case 10:
          tdc_x1_[ind2]->Fill(64+static_cast<int>( (*(tdce.dataBegin() + i_tdc)).channel) );
          break;
        case 11:
          tdc_y1_[ind2]->Fill(static_cast<int>( (*(tdce.dataBegin() + i_tdc)).channel ));
          break;
        case 12:
          tdc_y1_[ind2]->Fill(64 + static_cast<int>( (*(tdce.dataBegin() + i_tdc)).channel ));
          break;
        case 13:
          tdc_x1_[ind2]->Fill(static_cast<int>( (*(tdce.dataBegin() + i_tdc)).channel ));
          break;
        case 14:
          tdc_x1_[ind2]->Fill(64+static_cast<int>( (*(tdce.dataBegin() + i_tdc)).channel) );
          break;
        case 15:
          tdc_y1_[ind2]->Fill(static_cast<int>( (*(tdce.dataBegin() + i_tdc)).channel ));
          break;
        case 16:
          tdc_y1_[ind2]->Fill(64 + static_cast<int>( (*(tdce.dataBegin() + i_tdc)).channel ));
          break;
        default:
          cout << "tdc do not exist" << endl;
        }
*/
//std::cout<<" sort vector 1 "<<ind2<<std::endl;
	      }
		//filling histograms
//tdc_x1_[ind2]->Write();
//tdc_y1_[ind2]->Write();

//int hitsc=0;
//int hitsx=0;
//int hitsy=0;
//int hitsexp=0;
/*
int window=1;
bool hit1=false;
int hitc=0;
int time1=0;
int ch1=0;
int ch2=0;
int nr1=0;
int nr2=0;
int i_=0;
bool match=false;
bool run=true;
//std::cout<<" sort vector 2 "<<ind2<<std::endl;
while (hitc<100 && run) {
	//first plane
	if(tdchits[i_][0]==1 || tdchits[i_][0]==2 || tdchits[i_][0]==3 || tdchits[i_][0]==4){

		if(hit1==false) {
		time1=tdchits[i_][1];
		nr1=i_;	
		ch1=tdchits[i_][2];
		if(tdchits[i_][0]==2) ch1=ch1+64;
		hit1=true;
		}
		if(i_!=0){
			//STORE EACH HIT WITH WITH A TIME WINDOW
			//check i they fall within time window +/- window
			if((time1<=tdchits[i_][1]+window && time1>tdchits[i_][1]-window)&&i_!=nr1){
				nr2=i_;
				ch2=tdchits[i_][2];
				if(tdchits[i_][0]==4) ch2=ch2+64;

				//check if we're really matching x with y (eg. board 1 or 2 with 3 or 4)
				if(abs(nr2-nr1)>1) match=true;
				if((abs(nr2-nr1)==1)&&(nr1==2 || nr2==2) &&(nr1!=1 &&nr2!=1)) match=true;
				if(match==true &&(tdchits[nr1][0]!=tdchits[nr2][0])){
				std::cout<<" match "<<match<<" hit pl1 !! "<<hitc<<" ch1 "<<ch1<<" ch2 "<<ch2<< " hits size "<<int(tdchits.size())<<" time1 "<<tdchits[nr1][1]<<" time2 "<<tdchits[nr2][1]<<" board1 "<<tdchits[nr1][0]<<" board2 "<<tdchits[nr2][0]<<std::endl;

				 if(tdchits[nr1][0]!=tdchits[nr2][0]){
				 tdc_xy_[ind2]->Fill( ch1,ch2 );
				 hitc++;
					}
				match=false;
					}
				if((nr2>nr1)&&int(tdchits.size())>nr2){

				tdchits.erase(tdchits.begin()+nr2);
				tdchits.erase(tdchits.begin()+nr1-1);
				std::cout<<" hits erased nr2 greater "<<tdchits.size()<<" nr1 "<<nr1<<" nr2 "<<nr2<<std::endl;
					}
				if((nr1>nr2)&&int(tdchits.size())>nr1){

				tdchits.erase(tdchits.begin()+nr1);
				tdchits.erase(tdchits.begin()+nr2-1);
				std::cout<<" hits erased nr1 greater "<<tdchits.size()<<" nr1 "<<nr1<<" nr2 "<<nr2<<std::endl;
					}
				hit1=false;	
				}


			}

	
		}
i_++;
//std::cout<<i_<<std::endl;
if(i_==int(tdchits.size())) i_=0;
if(i_>400) run=false;
  }
*/

std::cout<<" wrote hits to file "<<std::endl;
	      std::cout << std::endl;
	      std::cout << "######################################################################" << std::endl;
	      std::cout << std::endl;
	      std::cout << std::endl;
	    }
	    break;
	  }
	default:
	  throw cet::exception( "Error in PawelDump TDCDump: unknown fragment type requested" );
	}
if(savexytdc){

	
	savexytdc=false;
	}
      }
//tdc_xy_[0]->Write();
tdc_timestamp_[0]->Write();
      std::cout << std::endl;

    }

tdchits.clear();
f2.Close();
}
//END of TDC

//WUT
  if ( raw.isValid() ) {

    if (raw->size() != 1 ) {
      throw cet::exception("Error in WUTDump module: expected one and only one spill fragment");
    }
   const auto& frag((*raw)[0]);
    TFile f3(histos_filename_.c_str(), "UPDATE");
    f3.cd();
    std::vector<const uint8_t*> lariatFragPtrsw;

    LariatFragment::LariatFragmentType lariatFragTypew = LariatFragment::LariatFragmentType::FRAGMENT_TYPE_WUT_HIT_HEADER;

    getLariatFragments(frag, 
     		       lariatFragTypew,
		       lariatFragPtrsw
		       );

    std::cout << std::endl;
    std::cout << "Run " << e.run() << ", subrun " << e.subRun()
       	      <<  " has " << lariatFragPtrsw.size()
   	      << " fragment(s) of type " << lariatFragTypew << std::endl;
      
    for (size_t idx = 0; idx < lariatFragPtrsw.size(); ++idx) {

      lariat::WUTHitFragment bb( lariatFragPtrsw[idx]  );

      bb.printHeader();

      if (max_wuts_to_show_ > 0) {

        uint32_t actual_to_show = max_wuts_to_show_ <= bb.total_values() ?
          max_wuts_to_show_ : bb.total_values() ;

	std::cout << std::endl;
	std::cout << "First " << actual_to_show
		  << " WUT values in the fragment: "
		  << std::endl;

	for (uint32_t i_wut = 0; i_wut < actual_to_show; ++i_wut) {
	  std::cout << "(Channel " << static_cast<int>( (*(bb.dataBegin() + i_wut)).channel ) <<
            ", Timebin " << (*(bb.dataBegin() + i_wut)).timeBin << ") ";
        }
      }

      std::cout << std::endl;
      std::cout << "######################################################################" << std::endl;
      std::cout << std::endl;
      std::cout << std::endl;
    }
f3.Close();
  } else {
    throw cet::exception("Error in Pawel Dump  WUTDump_module: expected one and only one spill fragment");
  }

  std::cout << std::endl;
//END OF WUT

}

DEFINE_ART_MODULE(lariat::PawelDump)
