////////////////////////////////////////////////////////////////////////
// Class:       Summary
// Module Type: analyzer
// File:        Summary_module.cc
//
// Generated at Wed Oct 22 11:47:26 2014 by John Freeman using artmod
// from cetpkgsupport v1_07_00.
////////////////////////////////////////////////////////////////////////

#include "art/Framework/Core/EDAnalyzer.h"
#include "art/Framework/Core/ModuleMacros.h"
#include "art/Framework/Principal/Event.h"
#include "art/Framework/Principal/Handle.h"
#include "art/Framework/Principal/Run.h"
#include "art/Framework/Principal/SubRun.h"
#include "art/Utilities/InputTag.h"
#include "fhiclcpp/ParameterSet.h"
#include "messagefacility/MessageLogger/MessageLogger.h"

#include "artdaq-core/Data/Fragments.hh"

#include "lariat-artdaq/Overlays/FragmentType.hh"
#include "lariat-artdaq/Overlays/GenericFragment.hh"

#include "LariatFragment.h"

#include "ModuleUtils.hh"

#include <vector>

namespace lariat {
  class Summary;
}

class lariat::Summary : public art::EDAnalyzer {
public:
  explicit Summary(fhicl::ParameterSet const & p);
  // The destructor generated by the compiler is fine for classes
  // without bare pointers or other resource use.

  // Plugins should not be copied or assigned.
  Summary(Summary const &) = delete;
  Summary(Summary &&) = delete;
  Summary & operator = (Summary const &) = delete;
  Summary & operator = (Summary &&) = delete;

  // Required functions.
  void analyze(art::Event const & e) override;


private:

  // Declare member data here.

};


lariat::Summary::Summary(fhicl::ParameterSet const & p)
  :
  EDAnalyzer(p)  // ,
 // More initializers here.
{}

void lariat::Summary::analyze(art::Event const & evt)
{

  art::EventNumber_t spillNumber = evt.event();

  art::Handle<artdaq::Fragments> raw;

  evt.getByLabel("daq", "SPILL", raw);

  if ( !raw.isValid() || raw->size() != 1) {

    throw cet::exception("Error in Summary_module: expected one and only one spill fragment");

  } else {

    const auto& spillfrag((*raw)[0]);

    std::vector<const uint8_t*> lariatFragPtrs;

    // First, grab the run number

    auto lariatFragType = 
      LariatFragment::LariatFragmentType::FRAGMENT_TYPE_RUNINFO ;

    getLariatFragments(spillfrag, lariatFragType, lariatFragPtrs);

    if (lariatFragPtrs.size() != 1) {
      std::cerr << "Found " << lariatFragPtrs.size() << " Run fragments" << std::endl;
      throw cet::exception("Error in WFViewer: expected one and only one Run fragment");
    }

    GenericFragment<LariatFragment::RunInfo> runInfo( lariatFragPtrs[0] );

    auto runNumber = runInfo.getHeader()->runNumber;

    // Now, count the different types of Lariat fragment

    std::vector<LariatFragment::LariatFragmentType> lariatFragTypes;
    lariatFragTypes.push_back( LariatFragment::LariatFragmentType::FRAGMENT_TYPE_CAEN_ADC );
    lariatFragTypes.push_back( LariatFragment::LariatFragmentType::FRAGMENT_TYPE_V1495 );
    lariatFragTypes.push_back( LariatFragment::LariatFragmentType::FRAGMENT_TYPE_TDC_CONTROLLER_HEADER );
    lariatFragTypes.push_back( LariatFragment::LariatFragmentType::FRAGMENT_TYPE_TDC_TDC_SPILL_HEADER );
    lariatFragTypes.push_back( LariatFragment::LariatFragmentType::FRAGMENT_TYPE_TDC_TDC_EVENT_HEADER );
    lariatFragTypes.push_back( LariatFragment::LariatFragmentType::FRAGMENT_TYPE_WUT_HIT_HEADER );

    for ( auto lariatFragType : lariatFragTypes) {

      lariatFragPtrs.clear();

      getLariatFragments(spillfrag,
			 lariatFragType,
			 lariatFragPtrs
			 );

      std::cout << std::endl;
      std::cout << "Run " << runNumber << ", subrun " << evt.subRun()
		<< ", spill " << spillNumber << " has " << lariatFragPtrs.size()
		<< " fragment(s) of type " << lariatFragType << std::endl;

    }
  }
}

DEFINE_ART_MODULE(lariat::Summary)
