
#include "GeneratorUtils.hh"

#include "art/Utilities/Exception.h"

#include <iostream>
#include <cstring>
#include <cmath>


namespace lariat {

  void writeLariatFragment(std::istream& lariatFragmentStream, 
			   artdaq::Fragment& artdaqFragment, 
			   std::size_t& lariatSize,
			   LariatFragment::LariatFragmentType& lariatType) {

    if ( artdaqFragment.dataSize() > 0) {
      throw cet::exception("Error in GeneratorUtils: artdaq fragment appears to already contain payload data");
    }

    std::streampos startpos = lariatFragmentStream.tellg(); 

    uint32_t dummy;

    lariatFragmentStream.read( reinterpret_cast<char*>(&dummy), 4);
    lariatSize = static_cast<std::size_t>( dummy );

    lariatFragmentStream.read( reinterpret_cast<char*>(&dummy), 4);
    lariatType = static_cast<LariatFragment::LariatFragmentType>( dummy );

    if (lariatType > 255) {
      throw cet::exception("Error in GeneratorUtils: unexpected lariat fragment type value has been read");
    }

    // Rewind to the beginning of the lariat fragment

    lariatFragmentStream.seekg( startpos );

    // Make sure memory for the lariat fragment is allocated in the artdaq fragment

    artdaqFragment.resize( ceil( lariatSize / static_cast<double>( sizeof(artdaq::RawDataType) ) ) );

    // And copy it over

    lariatFragmentStream.read( reinterpret_cast<char*>( &*artdaqFragment.dataBegin() ), lariatSize );


    artdaqFragment.setUserType( static_cast<artdaq::Fragment::type_t>( lariatType ) );
  }

}
