#ifndef lariat_artdaq_Generators_Lariat_hh
#define lariat_artdaq_Generators_Lariat_hh

// Lariat is a fragment generator providing a wrapper around an
// instance of the LariatReadout class

// Some C++ conventions used:

// -Append a "_" to every member variable

#include "fhiclcpp/fwd.h"
#include "artdaq-core/Data/Fragments.hh" 
#include "artdaq/Application/CommandableFragmentGenerator.hh"
#include "lariat-artdaq/Overlays/FragmentType.hh"

#include "include/LariatReadout.h"

#include <iostream>
#include <vector>
#include <string>

namespace lariat {    

  class Lariat : public artdaq::CommandableFragmentGenerator, private LariatReadout {
  public:
    explicit Lariat(fhicl::ParameterSet const & ps);

  private:

    char CheckKeypress();

    bool getNext_(artdaq::FragmentPtrs & output) override;

    std::string keyfilename_;

    // FHiCL-configurable variables. Note that the C++ variable names
    // are the FHiCL variable names with a "_" appended

    std::size_t const throttle_usecs_;
    std::size_t const throttle_usecs_check_;

    // End FHiCL-configurable variables
    
    std::vector<artdaq::Fragment::fragment_id_t> fragment_ids_; 

    artdaq::Fragment::sequence_id_t seqID_;

    // JCF, 1/9/15

    // Use of "should_stop()" renders transition overrides unnecessary

    // virtual void start() override;
    // virtual void stop() override;
    // virtual void pause() override;
    // virtual void resume() override;
    // virtual std::string report() override;

  };
}

#endif /* lariat_artdaq_Generators_Lariat_hh */
