#ifndef lariat_artdaq_Generators_SpillFileReader_hh
#define lariat_artdaq_Generators_SpillFileReader_hh

// SpillFileReader is a fragment generator designed to read in binary
// files in their entirety and pass them as payloads to artdaq
// fragments of type 10 ("spill" fragments)

// Some C++ conventions used:

// -Append a "_" to every private member function and variable

#include "fhiclcpp/fwd.h"
#include "artdaq-core/Data/Fragments.hh" 
#include "artdaq/Application/CommandableFragmentGenerator.hh"
//#include "lariat-artdaq/Overlays/CAENADCFragment.hh"
#include "lariat-artdaq/Overlays/FragmentType.hh"

// Would be nice to update makefiles to better locate Lariat's include directory...

#include <iostream>
#include <fstream>
#include "include/CAENDecoder.h"
#include "include/CAENConfiguration.h"


#include <random>
#include <vector>
#include <atomic>
#include <map>

namespace lariat {    

  class SpillFileReader : public artdaq::CommandableFragmentGenerator {
  public:
    explicit SpillFileReader(fhicl::ParameterSet const & ps);

    //  protected:
    //    const virtual std::string getConfigFilename() { return config_filename_; }

  private:

    bool getNext_(artdaq::FragmentPtrs & output) override;
    
    //std::vector<artdaq::Fragment::fragment_id_t> fragmentIDs_() override {
    //return fragment_ids_;
    //}

    // FHiCL-configurable variables. Note that the C++ variable names
    // are the FHiCL variable names with a "_" appended

    std::string input_filelist_;  

    // Sleep at start of each call to getNext_(), in us                
    std::size_t const throttle_usecs_;        

    // Period between checks for stop/pause during the sleep (must be
    // less than, and an integer divisor of, throttle_usecs_)
    
    std::size_t const throttle_usecs_check_;  

    // End FHiCL-configurable variables
    
    std::vector<artdaq::Fragment::fragment_id_t> fragment_ids_; 

    std::ifstream infilelist_;
    //    std::string output_directory_;

    std::ifstream infile_;



    bool new_file_requested_;
    //    std::streampos file_position_;


    // Should I investigate Boost's memory-mapped files (class
    // mapped_file, etc.) to deal with potentially huge input files?

    // Make sure bufferSize is large enough to hold a fragment...

    //    const std::size_t bufferSize_ = 1000000; 
    //    std::unique_ptr<char> buffer_;

    // Keep track of how many fragments in a given event we've seen;
    // until the Lariat binary files contain fragments with unique
    // board_ids within an event we'll just pick out the Nth fragment
    // in an event rather than looking for a fragment with board_id ==
    // N

    //    std::map <artdaq::Fragment::sequence_id_t, artdaq::Fragment::fragment_id_t> counted_fragments_;
    //    std::map <artdaq::Fragment::sequence_id_t, std::unique_ptr< artdaq::FragmentPtrs > > stored_fragments_;

    //    artdaq::Fragment::sequence_id_t currEvent_;
  };
}

#endif /* lariat_artdaq_Generators_SpillFileReader_hh */
