#ifndef lariat_artdaq_Overlays_GenericFragment_hh
#define lariat_artdaq_Overlays_GenericFragment_hh

#include "artdaq-core/Data/Fragment.hh"
#include "cetlib/exception.h"

#include <iostream>
#include <ostream>
#include <vector>

namespace lariat {
  template<typename HeaderType, typename ValType = uint8_t>
  class GenericFragment;

}

template<typename HeaderType, typename ValType>
class lariat::GenericFragment {

  public:


  typedef ValType val_t;


  // The constructor simply sets its const private member "fragPtr"
  // to refer to chunk of binary data pointed to it

  // It's up to the caller to make sure the pointer's actually
  // pointing at a CAEN fragment; eventually it would be nice to check
  // that this is, in fact, the case

  // JCF, 8/13/14 -- added a compile-time check

  GenericFragment(const uint8_t* const ptr ) : fragPtr_( ptr ) {

    // JCF, 8/13/14

    // The following check is designed to ensure that dataBegin()
    // correctly points to the beginning of where the fragment's
    // header ends and its array of ValTypes begins
    
    const std::size_t distanceToData = static_cast<std::size_t>( 
								reinterpret_cast<const uint8_t*>( dataBegin() ) -  
								reinterpret_cast<const uint8_t*>( getHeader() ) );


    if( distanceToData != sizeof( HeaderType)) {
      throw cet::exception( "Error in GenericFragment's pointer arithmetic; unexpected location in memory returned by dataBegin()" );
    }
  }

  const HeaderType* getHeader() const {

    return reinterpret_cast<const HeaderType*>( fragPtr_ );
  }

  virtual void printHeader() const {
    std::cout << "GenericFragment: printHeader() is designed to be overridden" << std::endl;
  }

  // The number of ADC values describing data beyond the header
  size_t total_values() const {
    return ( getHeader()->fragmentSize - sizeof( HeaderType ) ) / sizeof(val_t) ;
  }

  // Start of the ADC values, returned as a pointer to the ADC type
  val_t const * dataBegin() const {
    return reinterpret_cast<val_t const *>(getHeader() + 1);
  }

  // End of the ADC values, returned as a pointer to the ADC type
  val_t const * dataEnd() const {
    return dataBegin() + total_values();
  }

private:

  const uint8_t* const fragPtr_ ;

};

#endif /* lariat_artdaq_Overlays_GenericFragment_hh */
