#ifndef lariat_artdaq_Overlays_GenericWrapperFragment_hh
#define lariat_artdaq_Overlays_GenericWrapperFragment_hh

#include "include/RawBuffer.h"
#include "LariatFragment.h"

#include <iostream>
#include <vector>

// JCF, 12/9/14

// Implementation of "GenericWrapperFragment", an template of an
// artdaq::Fragment overlay class for Lariat fragments which have
// corresponding "fillFromBuffer" functions defined in
// lariat-online/daq/include/ and lariat-online/daq/src/

// Both the fragment class type and the fragment class header type
// will need to be supplied, e.g. 

// typedef GenericWrapperFragment<CAENFragment, CAENFragment::CaenHeader_t> CAENWrapperFragment ;

// Expectations are that the fragment class type has a function called
// "fillFromBuffer", and that the fragment class header type has
// uint32_t member variables "fragmentSize" and "fragmentType"

namespace lariat {
  
  template <typename FragmentClass, typename Header>
  class GenericWrapperFragment;
}

template <typename FragmentClass, typename Header>
class lariat::GenericWrapperFragment  {
  
  public:

  // Should I make this a ptr-to-char rather than ptr-to-uint8_t?

  GenericWrapperFragment(const uint8_t* const ptr ) : 
    lariat_frag_( new FragmentClass )
  {

    // Get the size of the fragment

    // A raw pointer, rather than a smart pointer, is used because the
    // artdaq fragment memory shouldn't be free-d

    const Header* fragment_class_header(reinterpret_cast<const Header*>( ptr ));

    uint32_t fragment_size = fragment_class_header->fragmentSize;
    uint32_t fragment_type = fragment_class_header->fragmentType;

    // Sanity check
    if (fragment_type > static_cast<uint32_t>(LariatFragment::LariatFragmentType::FRAGMENT_TYPE_N)) {
      throw cet::exception("Error in GenericWrapperFragment: fragment_type found to be nonsensical value of ") << fragment_type << std::endl;
    }

    // JCF, 10/31/14

    // "RawBuffer" doesn't accept ptr-to-const, as it uses the
    // streambuf::setg() function which doesn't accept ptr-to-const;
    // therefore I'll cast away the const-ness of the ptr returned by
    // getHeader()

    char* start_of_frag = const_cast<char*>( reinterpret_cast<const char*>(ptr) );

    RawBuffer raw_buffer(start_of_frag, fragment_size);
    std::istream in(&raw_buffer);

    lariat_frag_->fillFromBuffer(in);
  }


  const std::shared_ptr<const FragmentClass> GetFragment() {
    return lariat_frag_;
  }

private:
  
  std::shared_ptr<FragmentClass> lariat_frag_;

};

#endif /* lariat_artdaq_Overlays_GenericWrapperFragment_hh */
