#!/bin/bash


if [ $# != 2 ] ; then
     echo "Usage: $0 <original binary file> <root file containing artdaq::Fragment>"
     exit 1
fi

verbosity=0

origFile=$1
rootFile=$2
outFile="ExtractBinary.dat"  # The name of the file produced by the ExtractBinary module

if [[ ! -e $origFile || ! -e $rootFile ]] ; then
    echo "Couldn't find one or both of $origFile, $rootFile"
    exit 1
fi

fhiclFile=$LARIATARTDAQ_DIR/tools/fcl/RootFileReader.fcl

echo "Make sure that $fhiclFile has the ExtractBinary module defined in its path"

origFileSize=$(stat -c%s $origFile)

if [[ $verbosity > 0 ]] ; then
    art -s $rootFile -c $fhiclFile 
else
    art -s $rootFile -c $fhiclFile 2>&1 > /dev/null
fi

if ! [ -e $outFile ] ; then
    echo "No file named $outFile appears to have been produced"
    exit 2
fi

echo "Comparing $origFile against the first $origFileSize bytes of $outFile, extracted from $rootFile"
result=`cmp --bytes=$origFileSize  $origFile $outFile`

if [ "$result" == "" ]; then
    echo "Success: first $origFileSize bytes of the (uncompressed) artdaq::Fragment payload in $rootFile matches the bytes comprising $origFile"
else
    echo "Failure: mismatch between first $origFileSize bytes of the (uncompressed) artdaq::Fragment payload in $rootFile and the bytes comprising $origFile"
fi
