#!/bin/env python

# JCF, 9/25/14

# This script will, for a given run number, take the first N binary
# files for that run found in /daqdata, where N = max_files, and store
# them in an artdaq::Fragment object which in turn is stored in an
# Art-readable Root file which can be found in the directory defined
# by "outputdir". Not setting max_files at the command line
# effectively means that all files for the run will be processed.

# In order to tailor this for your work area, make sure to set the
# "basedir" and "outputdir" variables properly

import sys
import os
import glob
import re
import subprocess

if not 2 <= len(sys.argv) <= 3:
    sys.exit("Usage: " + sys.argv[0] + " <run number> (max files)")

run_number = sys.argv[1]

if len(sys.argv) >= 3:
    max_files = int(sys.argv[2])
else:
    max_files = 100000000


basedir = "/home/nfs/jcfree/lariat-online/daq"
outputdir = "/lariat/data/users/jcfree"
currentdir = os.getcwd()

filelist = basedir + "/lariat-artdaq/tools/fcl/datalist_combo.txt"
template_file = "wrapper.fcl.in"
fcl_file = "wrapper.fcl"

if not os.path.exists( basedir + "/setupLARIATARTDAQ"):
    print "Unable to find " + basedir + "/setupLARIATARTDAQ, exiting..."
    sys.exit(1)

proc = subprocess.Popen("ls -tr1 /daqdata/lariat_physics_" + run_number + "_*.dat", shell=True, 
                        stdout=subprocess.PIPE)

filenames = [filename.strip() for filename in proc.stdout.readlines()]
nfiles = len( filenames )

if nfiles == 0:
    print "Unable to find any files for requested run number " + run_number + ", exiting..."
    sys.exit(2)
else:
    print "Found " + str(nfiles) + " files"

if not os.path.exists( template_file ):
    print "Unable to find " + template_file + ", exiting..."
    sys.exit(3)

for i, filename in enumerate(filenames):

    if i >= max_files:
        break

    print "Processing " + filename + "..."
    res = re.search(r"_(\d+).dat$", filename)
    assert res, "Problem extracting subrun number from filename"
    subrun_number = res.group(1)

    inf = open( template_file )
    assert inf, "Problem opening " + template_file

    output_filename = outputdir + "/" + "lariat_r" + run_number + "_sr" + subrun_number + ".root"

    outf_filelist = open(filelist, "w")
    assert outf_filelist, "Unable to open " + str(filelist)

    outf_filelist.write( filename + "\n")
    outf_filelist.close()

    outf_fcl = open( fcl_file, "w")
    assert outf_fcl, "Unable to open " + str(fcl_file)

    for line in inf.readlines() :
        line = line.replace("RUN_NUMBER", run_number)
        #line = line.replace("SUB", subrun_number )
        line = line.replace("FILELIST", filelist)
        line = line.replace("OUTPUT_FILENAME", output_filename)
        outf_fcl.write( line )

    outf_fcl.close()

    cmds = []

    cmds.append( "source /grid/fermiapp/products/lariat/setup " )
    cmds.append( "cd " + basedir )
    cmds.append( "source ./setupLARIATARTDAQ" )
    
    cmds.append( "driver -c " + currentdir + "/" + fcl_file )

    status = subprocess.Popen( " ; ".join( cmds ) , shell=True, stdout=subprocess.PIPE).wait()

    if status != 0:
        raise Exception("Error in wrapper.py: problem launching process " + " ; ".join( cmds ) )

    
    
