#include "V1495Configuration.h"
#include <stdlib.h>
//#include <sstream>

V1495Configuration::V1495Configuration(std::string fileName):
  ConfigurationBase(fileName)
{
  bridgeType           = config.get<int>("V1495.bridgeType");
  link                 = config.get<int>("V1495.link");
  boardNumber          = config.get<int>("V1495.boardNumber");
  debugLevel           = config.get<uint32_t>("V1495.debugLevel");
  baseAddress          = config.get<uint32_t>("V1495.baseAddress");
  enableReadout        = config.get<bool>("V1495.enableReadout");
  nChan=config.get<uint32_t>("V1495.nchan");
  inputChannels.resize(nChan);
  for(uint i=0; i<nChan; i++){
    char buf[256];
    sprintf(buf,"V1495.in%i",i);
    std::string inx=buf;
    V1495InputChannel inchan;
    inchan.number=config.get<uint32_t>( inx+".number");
    std::string regstr=config.get<std::string>(inx+".reg");
    char** endptr=NULL;
    inchan.reg=strtol(regstr.c_str(),endptr,0);
    inchan.name=config.get<std::string>(inx+".name");
    inputChannels[i]=inchan;
  }

  nTrigPat=config.get<uint32_t>("V1495.ntrigpat");
  for(uint i=0; i<nTrigPat; i++){
    char buf[256]; sprintf(buf,"V1495.trigpat%i",i);
    std::string inx=buf;
    std::string irs=config.get<std::string>(inx+".inreg");
    //    std::string mrs=config.get<std::string>(inx+".maskreg");
    std::string crs=config.get<std::string>(inx+".cntreg");

    char** endptr=NULL;
    uint32_t inreg = strtol(irs.c_str(),endptr,0);
    //    uint32_t maskreg = strtol(mrs.c_str(),endptr,0);
    uint32_t maskreg = inreg;
    uint32_t cntreg = strtol(crs.c_str(),endptr,0);

    std::string ons=config.get<std::string>(inx+".on");    
    std::string offs=config.get<std::string>(inx+".off");    

    V1495TriggerPattern tp(inreg,maskreg,cntreg,inputChannels,ons,offs);
    triggerPatterns.push_back(tp);
  }

  nVetoPat=config.get<uint32_t>("V1495.nvetopat");
  for(uint i=0; i<nVetoPat; i++){
    char buf[256]; sprintf(buf,"V1495.vetopat%i",i);
    std::string inx=buf;
    std::string irs=config.get<std::string>(inx+".inreg");
    //    std::string mrs=config.get<std::string>(inx+".maskreg");
    std::string crs=config.get<std::string>(inx+".cntreg");

    char** endptr=NULL;
    uint32_t inreg = strtol(irs.c_str(),endptr,0);
    //    uint32_t maskreg = strtol(mrs.c_str(),endptr,0);
    uint32_t maskreg = inreg;
    uint32_t cntreg = strtol(crs.c_str(),endptr,0);

    std::string ons=config.get<std::string>(inx+".on");    
    std::string offs=config.get<std::string>(inx+".off");    

    V1495TriggerPattern tp(inreg,maskreg,cntreg,inputChannels,ons,offs);
    vetoPatterns.push_back(tp);
  }

  char ** endptr=NULL;

  triggerDelay=config.get<uint32_t>("V1495.delay.ticks");
  std::string tdrs=config.get<std::string>("V1495.delay.reg");
  triggerDelayReg=strtol(tdrs.c_str(),endptr,0);

  vetoLength=config.get<uint32_t>("V1495.veto.ticks");
  std::string vlrs=config.get<std::string>("V1495.veto.reg");
  vetoLengthReg=strtol(vlrs.c_str(),endptr,0);
  
  std::string crs=config.get<std::string>("V1495.control.reg");
  controlReg=strtol(crs.c_str(),endptr,0);
  counterWidth=config.get<uint32_t>("V1495.control.cwidth");
  triggerWidth=config.get<uint32_t>("V1495.control.twidth");
  M=config.get<uint32_t>("V1495.control.M");

  std::string tvrs=config.get<std::string>("V1495.tvmask.reg");
  tvMaskReg=strtol(tvrs.c_str(),endptr,0);
  std::string tvs=config.get<std::string>("V1495.tvmask.mask");
  tvMask=strtol(tvs.c_str(),endptr,0);
  std::cout << " Finished reading V1495 configuration" << std::endl;
}


void V1495Configuration::print(std::ostream& os) 
{
  os << "\n========= V1495 Configuration =========" << std::endl;
  ConfigurationBase::print(os);
  os << "  BridgeType             " << bridgeType       << std::endl;
  os << "  Link                   " << link             << std::endl;
  os << "  BoardNumber            " << boardNumber      << std::endl;
  os << "  DebugLevel             " << debugLevel       << std::endl;
  os << "  BaseAddress                0x" << std::hex << baseAddress <<
    " (" << std::dec << baseAddress << std::endl;
  
  os<<"\n========= Input Channels =========="<<std::endl;
  V1495InputChannels::const_iterator it 
    = inputChannels.begin();
  for(;it!=inputChannels.end(); it++){
    it->print(os);
  }
  os<<"\n======= Trigger Patterns =========="<<std::endl;
  for(uint i=0; i<triggerPatterns.size(); i++){
    os<<"*** Pattern "<<i<<" ***"<<std::endl; 
    triggerPatterns[i].print(os);
  }
  os<<"\n======= Veto Patterns =========="<<std::endl;
  for(uint i=0; i<vetoPatterns.size(); i++){
    os<<"*** Pattern "<<i<<" ***"<<std::endl; 
    vetoPatterns[i].print(os);
  }
  os<<"\n========== Trigger & Veto ==========="<<std::endl;
  os<<"Trigger Delay     @ 0x"<<std::hex<<triggerDelayReg<<" : "<<std::dec<<triggerDelay<<" ticks"<<std::endl;
  os<<"Veto Length       @ 0x"<<std::hex<<vetoLengthReg<<" : "<<std::dec<<vetoLength<<" ticks"<<std::endl;
  os<<"Trigger/Veto Mask @ 0x"<<std::hex<<tvMaskReg<<" : 0x"<<tvMask<<std::dec<<std::endl;

  os<<"\n======= Control Register ========="<<std::endl;
  os<<"Control Register @ 0x"<<std::hex<<controlReg<<std::dec<<std::endl;
  os<<"Counter output width : "<<counterWidth<<" ticks "<<std::endl;
  os<<"Trigger output width : "<<triggerWidth<<" ticks "<<std::endl;
  os<<"M of N coincidence   : "<<M<<" of "<<4<<std::endl;

}


V1495TriggerPattern::V1495TriggerPattern(uint32_t ir, uint32_t mr,uint32_t cr,
					 const V1495InputChannels& chans,
					 std::string ons, std::string offs): inreg(ir), maskreg(mr),cntreg(cr),counts(0),bitpattern(0x0),bitmask(0xFFFF),onstring(ons), offstring(offs)
{

  configureBitPatterns(chans,onstring,offstring);
  

}

void V1495TriggerPattern::configureBitPatterns(const V1495InputChannels& chans,
					       std::string onstring, 
					       std::string offstring){
  
  // reinitialize the pattern and mask
  bitpattern=std::bitset<32>(0x0);
  bitmask=std::bitset<32>(0x0); // mask all channels out by default

  // loop over the input channels
  // look for an occurence of that input channel
  //    in onstring or offstring
  //    if so, then set the appropriate bit
  //    in bitpattern accordingly and enable that
  //    channel in bitmask
  // Unmentioned channels are treated as "don't care"
  // their place in bitpattern will be set to zero
  // and a zero will also appear in bitmask to mask them out
  
  V1495InputChannels::const_iterator it = chans.begin();
  for(;it!=chans.end(); it++){
    std::string chan_name=it->name;
    uint32_t chan_num=it->number;
    if(onstring.find(chan_name)!=std::string::npos){
      bitpattern.set(chan_num,true);
      bitmask.set(chan_num,true);
    }
    else if(offstring.find(chan_name)!=std::string::npos){
      bitpattern.set(chan_num,false);
      bitmask.set(chan_num,true);
    }
  }  
}


void V1495TriggerPattern::print(std::ostream& os)  {
  using std::endl;
  using std::hex;
  using std::dec;
  os<<"input register  : 0x"<<hex<<inreg<<endl;
  //  os<<"mask register   : 0x"<<hex<<maskreg<<endl;
  os<<"counts register : 0x"<<hex<<cntreg<<endl;
  os<<"Required ON: "<<onstring<<endl;
  os<<"Required OFF: "<<offstring<<endl;
  os<<"Bit pattern required : "<<bitpattern<<endl;
  os<<"Bit mask required    : "<<bitmask<<endl;
  os<<"Resulting logic      : ";
  for(int i=bitpattern.size()-1; i>=0; i--){
    if(bitmask[i]==true && bitpattern[i]==true){ os<<'Y';}
    else if(bitmask[i]==true && bitpattern[i]==false){ os<<'N';}
    else if(bitmask[i]==false){os<<'-';}
  }
  os<<dec<<endl;
}
