#include "V1495Driver.h"

#include <iostream>
#include <unistd.h>

using namespace caen;


V1495Driver::V1495Driver(const LariatConfiguration* lariat_config_, 
			 V1495Configuration* config_, 
			 caen::vme_handle h_):config(config_){
  
#ifdef USEVMELIB
  h=h_;
  baddr=config->baseAddress;
  fwVersion(fwX,fwY);
  std::cout<<"CAEN 1495 at 0x"<<std::hex<<baddr<<std::dec
	   <<" FW "<<fwX<<"."<<fwY<<std::endl; 

#else 
  CAENComm_ConnectionType connectionType=static_cast<CAENComm_ConnectionType>(config->bridgeType);
  CAENComm_ErrorCode err = 
    CAENComm_OpenDevice(connectionType,config->link,
			config->boardNumber,config->baseAddress,&h);
 
 if( err != CAENComm_Success){
   char buf[256]; CAENComm_DecodeError(err,buf);
   std::string huh="1495 constructor: "; huh+= buf;
   throw caen_exception(huh);
 }
 
 
 baddr=config->baseAddress;
 fwVersion(fwX,fwY);
 std::cout<<"CAEN 1495 at 0x"<<std::hex<<baddr<<std::dec
	  <<" FW "<<fwX<<"."<<fwY<<std::endl; 
#endif

  

  

}

#ifdef USEVMELIB
V1495Driver::V1495Driver(caen::vme_handle handle, uint32_t base_address){
  h=handle;
  baddr=base_address;
  fwVersion(fwX,fwY);
  std::cout<<"CAEN 1495 at 0x"<<std::hex<<baddr<<std::dec
	   <<" FW "<<fwX<<"."<<fwY<<std::endl; 
}

#else 
V1495Driver::V1495Driver(uint32_t base_address, int linknum, int board,
		     CAENComm_ConnectionType linktype)
{
  CAENComm_ErrorCode err = 
    CAENComm_OpenDevice(linktype,linknum,board,base_address,&h);

  if( err != CAENComm_Success){
    char buf[256]; CAENComm_DecodeError(err,buf);
    std::string huh="1495 constructor: "; huh+= buf;
    throw caen_exception(huh);
  }

    
  baddr=base_address;
  fwVersion(fwX,fwY);
  std::cout<<"CAEN 1495 at 0x"<<std::hex<<baddr<<std::dec
	   <<" FW "<<fwX<<"."<<fwY<<std::endl; 
  // 

}
#endif


V1495Driver::~V1495Driver(){

#ifdef USEVMELIB
  // we don't need to do anything
#else  
  CAENComm_ErrorCode err = CAENComm_CloseDevice(h);
  if( err != CAENComm_Success){
    char buf[256]; CAENComm_DecodeError(err,buf);
    std::string huh="1495 destructor: "; huh+= buf;
    throw caen_exception(huh);
  }
#endif

}

void V1495Driver::fwVersion(uint32_t& X, uint32_t& Y){
  const uint32_t reg=0x800C;// 1495 manual
  uint32_t data;

#ifdef USEVMELIB
  CVErrorCodes err = CAENVME_ReadCycle(h,baddr+reg,(void*) &data,cvA32_U_DATA,cvD32);
  if( err != cvSuccess){    
    std::string huh="1495 fwVersion: "; huh+=decode_error(err);;
    throw caen_exception(huh);
  }
#else
  CAENComm_ErrorCode err=CAENComm_Read32(h,reg,&data);
  if( err != CAENComm_Success){
    char buf[256]; CAENComm_DecodeError(err,buf);
    std::string huh="1495 fwVersion: "; huh+= buf;
    throw caen_exception(huh);
  }
#endif

  X=bitgrab(data,8,15);
  Y=bitgrab(data,0,7);

}

void V1495Driver::reset(){

  uint32_t cregval = readRegister(config->controlReg); // our user defined control register
  std::cout<<"V1495Driver::reset():  before reset control Register Reads: "<<std::hex<<cregval<<std::dec<<std::endl;
  // This will issue a reset without changing the 
  // channel pulse width or trigger width
  // it just sets counters to zero
  // first we set a 0 on bit zero
  // this asserts the reset
  writeRegister(config->controlReg,cregval&0xFFFFFFFE);
  // then we set a 1 on bit zero
  // this frees it again
  writeRegister(config->controlReg,cregval|0x00000001);

  // NOTE: longstanding confusion. For the longest time
  // I could not understand how the reset register worked.
  // If you look at the VHDL code, it certainly seems apparent
  // that writing a 0 to the register above - REG_RW1(0) -
  // should assert the reset. And indeed, it does.
  // Yet, holding the NIM input on the lowest connector 
  // - which maps to register D(2) - at 0V does not do a reset.
  // Instead asserting a NIM logical high (-700mV) resets.
  // WHAT GIVES?!  --> When used as inputs, the LEMOS on the 1495 are inverted.
  
 
  cregval = readRegister(config->controlReg);
  std::cout<<"V1495Driver::reset():  after reset control Register Reads: "<<std::hex<<cregval<<std::dec<<std::endl;


}


void V1495Driver::reloadFirmware(){

  const uint32_t reg=0x8016; // 1495 manual
  uint32_t data=0x1;

#ifdef USEVMELIB
  CVErrorCodes err = CAENVME_WriteCycle(h,baddr+reg,(void*) &data,cvA32_U_DATA,cvD32);
  if(err != cvSuccess){
    std::string huh="1495 reload_firmware() "; huh+= decode_error(err);
    throw caen_exception(huh);
  }
#else
  CAENComm_ErrorCode err=CAENComm_Write32(h,reg,data);
  if( err != CAENComm_Success){
    char buf[256]; CAENComm_DecodeError(err,buf);
    std::string huh="1495 reloadFirmware() "; huh+= buf;
    throw caen_exception(huh);
  }
#endif  
  
}

uint32_t V1495Driver::readRegister(uint32_t reg){

  uint32_t data;

#ifdef USEVMELIB

  CVErrorCodes err = CAENVME_ReadCycle(h,baddr+reg,(void*) &data,cvA32_U_DATA,cvD32);
  if( err != cvSuccess){    
    std::string huh="1495 readRegister: "; huh+=decode_error(err);;
    throw caen_exception(huh);
  }

#else

  CAENComm_ErrorCode err=CAENComm_Read32(h,reg,&data);
  if( err != CAENComm_Success){
    char buf[256]; CAENComm_DecodeError(err,buf);
    std::string huh="1495 readRegister: "; huh+= buf;
    throw caen_exception(huh);
  }
#endif

  return data;
}

void V1495Driver::writeRegister(uint32_t reg, uint32_t data){

#ifdef USEVMELIB
  CVErrorCodes err = CAENVME_WriteCycle(h,baddr+reg,(void*) &data,cvA32_U_DATA,cvD32);
  if(err != cvSuccess){
    std::string huh="1495 writeRegister() "; huh+= decode_error(err);
    throw caen_exception(huh);
  }
#else
  CAENComm_ErrorCode err=CAENComm_Write32(h,reg,data);
  if( err != CAENComm_Success){
    char buf[256]; CAENComm_DecodeError(err,buf);
    std::string huh="1495 writeRegister: "; huh+= buf;
    throw caen_exception(huh);
  }
#endif

}

void V1495Driver::configureControlRegister(){

  // configure control register
  // setting a 1 on bit 1 prevents a reset
  uint32_t cregval=0x1;
  // counterWidth to bits 1 and 2
  cregval|=bitpack(config->counterWidth,0,1,1);
  // triggerWidth to bits 3 and 4
  cregval|=bitpack(config->triggerWidth,0,1,3);
  // M to bits 5-7
  cregval|=bitpack(config->M,0,2,5);
  writeRegister(config->controlReg,cregval);
  usleep(1000);
  uint32_t readback=readRegister(config->controlReg);
  if(readback!=cregval){
    char buf[256]; sprintf(buf,"configureControlRegister: readback test failed after write to 0x%X. Wrote 0x%X , Read 0x%X",config->controlReg,cregval,readback);
    throw caen_exception(buf,cvGenericError);
  }
}

void V1495Driver::configureCoincidence(const V1495TriggerPattern& trigpat){

  uint32_t val=0x0;
  // bits 0-15 are the coincidence pattern
  uint32_t pattern=trigpat.bitpattern.to_ulong();
  val|=bitpack(pattern,0,15,0);
  // bits 16-31 are the bitmask
  uint32_t mask=trigpat.bitmask.to_ulong();
  val|=bitpack(mask,0,15,16);
  writeRegister(trigpat.inreg,val);
  usleep(1000);
  uint32_t readback= readRegister(trigpat.inreg);
  if(readback!=val){
    char buf[256]; sprintf(buf,"configureCoincidence: readback test failed after write to 0x%X. Wrote 0x%X , Read 0x%X",trigpat.inreg,val,readback);
    throw caen_exception(buf,cvGenericError);
  }
}

void V1495Driver::configureDelays(){

  writeRegister(config->triggerDelayReg,config->triggerDelay);
  uint32_t readback=readRegister(config->triggerDelayReg);
  if(readback!=config->triggerDelay){
    char buf[256]; sprintf(buf,"configureDelays: readback test failed after write to 0x%X. Wrote 0x%X , Read 0x%X",config->triggerDelayReg,config->triggerDelay,readback);
    throw caen_exception(buf,cvGenericError);
  }

  writeRegister(config->vetoLengthReg,config->vetoLength);
  readback=readRegister(config->vetoLengthReg);
  if(readback!=config->vetoLength){
    char buf[256]; sprintf(buf,"configureDelays: readback test failed after write to 0x%X. Wrote 0x%X , Read 0x%X",config->vetoLengthReg,config->vetoLength,readback);
    throw caen_exception(buf,cvGenericError);
  }


}

void V1495Driver::configureCommon(){

  configureControlRegister();

  configureDelays();

  // input channels need no configuration
  
  for(uint i=0; i<config->nTrigPat; i++){
    usleep(1000);
    std::cout<<"Configuring trigger pattern "<<i<<std::endl;
    try{
      configureCoincidence(config->triggerPatterns[i]);
    } catch (caen_exception& ex){
      std::cout<<"Caught exception "<<ex.what()<<std::endl;
      std::cout<<"Will try to carry on"<<std::endl;
    }
  }

  for(uint i=0; i<config->nVetoPat; i++){
    usleep(1000);
    std::cout<<"Configuring veto pattern "<<i<<std::endl;
    try{
      configureCoincidence(config->vetoPatterns[i]);
    } catch (caen_exception& ex){
      std::cout<<"Caught exception "<<ex.what()<<std::endl;
      std::cout<<"Will try to carry on"<<std::endl;
    }
  }
  
}


void V1495Driver::print(std::ostream& os){
  
  config->print(os);
  frag.print(os);

}

void V1495Driver::readEvent(){

  // make a new fragment from scratch
  frag = V1495Fragment();
  
  // read/fill header information
  frag.header.nChan = config->nChan;
  frag.header.nTrigPat = config->nTrigPat;
  frag.header.nVetoPat = config->nVetoPat;
  frag.header.M = config->M;
  
  // read channel information  
  for(uint i=0; i<config->nChan; i++){
    V1495InputChannel& cchan = config->inputChannels[i];
    V1495Fragment::V1495ChannelData_t fchan;
    for(uint j=0; j<sizeof(fchan.name); j++){fchan.name[j]=0;}
    cchan.name.copy(fchan.name,sizeof(fchan.name));
    fchan.num=cchan.number;
    cchan.counts = readRegister(cchan.reg);
    fchan.counts=cchan.counts;
    frag.channelData.push_back(fchan);
  }
  
  // read trigger pattern information
  for(uint i=0; i<config->nTrigPat; i++){
    V1495TriggerPattern& cpat=config->triggerPatterns[i];
    V1495Fragment::V1495TriggerPatternData_t fpat;
    fpat.num=i;
    fpat.pattern=readRegister(cpat.inreg);
    cpat.counts=readRegister(cpat.cntreg);
    fpat.counts=cpat.counts;
    frag.triggerPatternData.push_back(fpat);    
  }

  // read veto pattern information
  for(uint i=0; i<config->nVetoPat; i++){
    V1495TriggerPattern& cpat=config->vetoPatterns[i];
    V1495Fragment::V1495TriggerPatternData_t fpat;
    fpat.num=i;
    fpat.pattern=readRegister(cpat.inreg);
    cpat.counts=readRegister(cpat.cntreg);
    fpat.counts=cpat.counts;
    frag.vetoPatternData.push_back(fpat);    
  }
  
}

void V1495Driver::readEvent(std::ostream * os){
  readEvent();
 
  frag.streamToBuffer(*os);  
}


caen::vme_handle V1495Driver::getVMEHandle(){
#ifdef USEVMELIB
  std::cout<<"I'm using the VMELIB"<<std::endl;
  return h;
#else 
  std::cout<<"I'm using CAENComm"<<std::endl;
  caen::vme_handle vmehandle;
  
  // BJR 10/29/14 unused variable commented out
  // void* vptr = static_cast<void*>(&vmehandle);
  //  CAENComm_ErrorCode err = CAENComm_Info(h, CAENComm_VMELIB_handle,vptr);
  CAENComm_ErrorCode err = CAENComm_Info(h, CAENComm_VMELIB_handle,&vmehandle);
  if(err!=CAENComm_Success){
    char mess[10000];
    CAENComm_DecodeError(err, mess);
    std::string huh="V1495Driver::getVMEHandle() : ";
    huh+=mess;
    throw caen::caen_exception(huh,err);
  }
  return vmehandle;
#endif 
  
}
